/*
 * frilogos.h
 * This file is part of FriLogos
 *
 * Copyright (C) 2012 - Alexandre Dussart
 *
 * FriLogos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * FriLogos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FriLogos. If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef __FRILOGOS_H__
#define __FRILOGOS_H__


#include <Eet.h>
#include <Ecore.h>
#include <Ecore_File.h>
#include <Edje.h>
#include <Elementary.h>


#ifdef HAVE_EVIL
#include <Evil.h>
#include <windows.h>
#include <shlobj.h>
#include <commdlg.h>
#endif


#ifdef HAVE_EMOTION
#include <Emotion.h>
#endif


#include "libfrilogos.h"


//
//Define App UI. Can be used to modify it dynamicaly.
//
typedef struct
{
	Evas_Object *win;
	Evas_Object *toolbar;
	Evas_Object *modules_list;
	Evas_Object *naviframe;
	Evas_Object *description_text;
	
	//Notifying when an event occurs.
	Evas_Object *bubble;
	Evas_Object *bubble_ic;
	Evas_Object *bubble_label;

	//Contains some current selection.
	Module_Info *module_selection;
	Patient_Info *patient_selection;
} App_Info;



//about.c: About dialog box.
extern void about_dialog_new(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__);

//TODO: make init.c and shutdown.c funcs.
//init.c: Database & data handling functions./
extern int frilogos_init();

//items_browser.c: Items browser.
extern void items_browser_new(void *data, Evas_Object *obj, void *event_info);

//patients_browser.c: Patients browser.
extern void patients_browser_new(void *data, Evas_Object *obj, void *event_info);

//stats_browser.c: Score/Statistics functions gui.
extern void stats_browser_new(App_Info *app);

//Win32 ported funcs.
#ifdef HAVE_EVIL
extern char *efreet_data_home_get();
#endif

#endif /* __FRILOGOS_H__ */
