/*
 * import_dlg.c
 * This file is part of FriLogos
 *
 * Copyright (C) 2012 - Alexandre Dussart
 *
 * FriLogos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * FriLogos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FriLogos. If not, see <http://www.gnu.org/licenses/>.
 */


#include "import_dlg.h"

//Taken from glib-2.0 credits goes to his authors.
static const char *_user_home_get(void);


//Return home user dir in a portable way.
static const char *_user_home_get(void)
{
#ifdef G_OS_WIN32
	char *home;
	home = g_strdup (g_getenv ("HOME"));

	if(!home)
	{
	  free (home);
	  home = NULL;
	}

	if (home)
	{
      		char *p;
		while ((p = strchr (home, '/')) != NULL)
		*p = '\\';
    	}

	if (!home)
	{
		if (g_getenv ("USERPROFILE") != NULL)
		home = g_strdup (g_getenv ("USERPROFILE"));
	}

	if (!home)
		home = get_special_folder (CSIDL_PROFILE);

    return home;
#else
    return(getenv ("HOME"));
#endif
}



static void _fs_selected_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info)
{
	ImportItemSelector *iis = (ImportItemSelector *) data;
   
	elm_object_text_set(iis->filename_entry, event_info);

    elm_image_file_set(iis->preview_img, "", NULL);
    elm_object_text_set(iis->infos_entry, "");

    if((eina_str_has_extension(event_info, ".jpg") == EINA_TRUE) ||
    (eina_str_has_extension(event_info, ".jpeg") == EINA_TRUE) ||    
    (eina_str_has_extension(event_info, ".gif") == EINA_TRUE) ||                    
    (eina_str_has_extension(event_info, ".bmp") == EINA_TRUE))
    {
        elm_image_file_set(iis->preview_img, event_info, NULL);
    }
    else if((eina_str_has_extension(event_info, ".eet") == EINA_TRUE))
    {
        Item_Info *item;
        item = item_read(event_info);
        if(item)
        {  

            char s[512];
            elm_image_file_set(iis->preview_img, event_info, "images/0");
        
            snprintf(s, sizeof(s),    _("Name:%s<br>"
                                    "Group:%s<br>"
                                    "Subgroup:%s<br>"
                                    "Created with:FriLogos %s<br>"
                                   "Creation Date:%s<br>"
                                   "Revision Date:%s<br>"),
                                   item->name,
                                   item->group, 
                                   item->subgroup,
                                   item->version,
                                   item->creation_date,
		                           item->revision_date);
        elm_object_text_set(iis->infos_entry, s);

        item = item_free(item);
        }
    }
}


//
//Close a window by deleting evas object passing arg.
//
static void _cancel_bt_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
    importitemselector_close(data);
};


//
//
//
void importitemselector_close(ImportItemSelector *iis)
{
    evas_object_del(iis->win);
    free(iis);
}


//
//
//
ImportItemSelector *importitemselector_add()
{
	ImportItemSelector *iis;
	Evas_Object *ic;

    iis = malloc(sizeof(*iis));

    iis->win = elm_win_util_standard_add("myfileselector", _("Import a FriLogos item..."));
    elm_win_center(iis->win, EINA_TRUE, EINA_TRUE);
	elm_win_autodel_set(iis->win, EINA_TRUE);

    Evas_Object *gd = elm_grid_add(iis->win);
    elm_grid_size_set(gd, 100, 100);
    elm_win_resize_object_add(iis->win, gd);
    evas_object_size_hint_weight_set(gd, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_show(gd);
	    
 
    iis->fs = elm_fileselector_add(iis->win);    
    elm_fileselector_buttons_ok_cancel_set(iis->fs, EINA_FALSE);
    elm_fileselector_is_save_set(iis->fs, EINA_FALSE);
    elm_fileselector_path_set(iis->fs, _user_home_get());
    evas_object_size_hint_weight_set(iis->fs, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(iis->fs, EVAS_HINT_FILL, EVAS_HINT_FILL);
    elm_grid_pack(gd, iis->fs , 1, 1, 50, 80);
    evas_object_show(iis->fs);
    evas_object_smart_callback_add(iis->fs, "selected", _fs_selected_cb, iis);


    //Set preview panel.
    iis->preview_img =  elm_image_add(iis->win);    
    elm_image_smooth_set(iis->preview_img, EINA_TRUE);
    elm_image_aspect_fixed_set(iis->preview_img, EINA_TRUE);
    evas_object_size_hint_weight_set(iis->preview_img, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_show(iis->preview_img);
    
    iis->preview_frame= elm_frame_add(iis->win);
    elm_object_text_set(iis->preview_frame, _("Preview"));
    elm_object_content_set(iis->preview_frame, iis->preview_img);
    elm_grid_pack(gd, iis->preview_frame, 55, 10, 40, 30);    
    evas_object_show(iis->preview_frame);
    
    iis->infos_entry = elm_entry_add(iis->win);
    elm_entry_editable_set(iis->infos_entry, EINA_FALSE);
    elm_entry_scrollable_set(iis->infos_entry , EINA_TRUE);
    elm_entry_single_line_set(iis->infos_entry , EINA_FALSE);
    elm_grid_pack(gd, iis->infos_entry , 55, 40, 40, 40);
    evas_object_show(iis->infos_entry );
    //End of preview panel.

    //Show cucrent selected file.
    iis->filename_entry = elm_entry_add(iis->win);
    elm_entry_scrollable_set(iis->filename_entry, EINA_TRUE);
    elm_entry_single_line_set(iis->filename_entry, EINA_TRUE);
    elm_grid_pack(gd, iis->filename_entry, 0, 83, 100, 7);
    evas_object_show(iis->filename_entry);
    
    //Buttons bar.
    iis->action_bt =elm_button_add(iis->win);
    elm_object_text_set(iis->action_bt,  _("Import"));
	ic = elm_icon_add(iis->win);
   	elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
   	elm_icon_standard_set(ic, "document-open");
   	elm_object_part_content_set(iis->action_bt, "icon", ic);
    elm_grid_pack(gd, iis->action_bt , 35, 92, 15, 7);   	
    evas_object_show(iis->action_bt);
    
    iis->cancel_bt =elm_button_add(iis->win);
	elm_object_text_set(iis->cancel_bt,  _("Close"));
	ic = elm_icon_add(iis->win);
   	elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
	elm_icon_standard_set(ic, "window-close");
   	elm_object_part_content_set(iis->cancel_bt, "icon", ic);
    elm_grid_pack(gd, iis->cancel_bt , 50, 92, 15, 7);
    evas_object_show(iis->cancel_bt );
	evas_object_smart_callback_add(iis->cancel_bt, "clicked", _cancel_bt_clicked_cb, iis);    
    //End of buttons bar.

    ///Forcing min height.
    evas_object_resize(iis->win, 600, 500);
    evas_object_show(iis->win);        

	return iis;
}
