/*
 * items_browser.c
 * This file is part of FriLogos
 *
 * Copyright (C) 2012 - Alexandre Dussart
 *
 * FriLogos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * FriLogos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FriLogos. If not, see <http://www.gnu.org/licenses/>.
 */


#include "frilogos.h"
#include "myfileselector.h"
#include "import_dlg.h"

typedef struct
{
	Evas_Object *win;

    //Notification.
	Evas_Object *bubble;
	Evas_Object *bubble_txt;;
	Evas_Object *bubble_ic;
	Ecore_Timer *bubble_timer;
    //Browser Widgets.
	Evas_Object *ggrid;
    Evas_Object *item_desc_entry;
    Evas_Object *groups_hs;

    Elm_Gengrid_Item_Class *gic;
    Elm_Gengrid_Item_Class *ggic;

	//Eina_List *items;
    Group_Info *group_selected;

	App_Info *app;
} Items_Browser;

static Items_Browser *browser = NULL;


//Items Browser local functions.
static void _browser_title_update();
static void _browser_notify_set(const char *s, const char *ic);

//Widgets callbacks.
static void _close_bt_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__);
static void _add_bt_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__);
static void _groups_hs_selected_cb(void *data, Evas_Object *obj, void *event_info);
static void _add_group_bt_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__);
static void _new_group_bt_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__);
static void _add_subgroup_bt_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__);
static void _new_subgroup_bt_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__);
static void _description_ctxitem_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__);
static void _remove_ctxitem_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__);
static void _export_ctxitem_clicked_cb(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__);
static void _import_bt_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__);

//static void _size_sl_changed_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__);

//GenGrid specific callbacks.
static void _ggrid_selected_cb(void *data, Evas_Object *obj, void *event_info);
static void _ggrid_moved_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info);
static void _ggrid_del(void *data, Evas_Object *obj __UNUSED__);
static char * _ggrid_subgroup_text_get(void *data, Evas_Object *obj __UNUSED__, const char *part __UNUSED__);
static char * _ggrid_item_text_get(void *data, Evas_Object *obj __UNUSED__, const char *part __UNUSED__);
static Evas_Object* _ggrid_content_get(void *data, Evas_Object *obj, const char *part);
static Eina_Bool _ggrid_state_get(void *data __UNUSED__, Evas_Object *obj __UNUSED__, const char *part __UNUSED__);
static void _ggrid_clickeddouble_cb(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info);


//
//Close browser and freeing some allocated stuff.
//
static void _close_bt_clicked_cb(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
    ecore_timer_del(browser->bubble_timer);
    browser->group_selected = group_free(browser->group_selected);
    elm_gengrid_item_class_ref(browser->gic);
    elm_gengrid_item_class_free(browser->gic );
    elm_gengrid_item_class_ref(browser->ggic);
    elm_gengrid_item_class_free(browser->ggic );
	evas_object_del(browser->win);
	
	free(browser);
	browser = NULL;
}

static void _import_action_bt_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
    ImportItemSelector *iis = data;
    char s[256];
    Elm_Object_Item *it;
    Item_Info *item;

    item = item_read(elm_fileselector_selected_get(iis->fs));

    if(!item)
        return;

	item_import(item);

    it = elm_gengrid_item_append(browser->ggrid, browser->gic, item, _ggrid_selected_cb, browser);
    elm_gengrid_item_update(it);
    
    snprintf(s, sizeof(s), _("Item %s has been imported."), item->name);
    _browser_notify_set(s, "info");
    
    importitemselector_close(iis);
}



//
//
//
static void _import_bt_clicked_cb(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
 	ImportItemSelector *iis;
 	
	iis = importitemselector_add();
    evas_object_smart_callback_add(iis->action_bt, "clicked", _import_action_bt_clicked_cb, iis);
}



//
//
//
static void _add_group_bt_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
    const char *buf;
    char s[128];
	Evas_Object *entry = data;
    Group_Info *group;

	buf = elm_object_text_get(entry);
    if(strlen(buf) < 1)
    {
        _browser_notify_set(_("Group name can't be null."), "dialog-warning");
        return;
    }

	group = group_new();
	group_name_set(group, buf);
	group->subgroups = NULL;

    //Add new group to hoversel widget.
	elm_hoversel_item_add(browser->groups_hs, elm_entry_markup_to_utf8(group->name), NULL, ELM_ICON_NONE, NULL, group);
    elm_hoversel_hover_begin(browser->groups_hs);

    snprintf(s, sizeof(s), _("New %s group has been added."), group->name);
    _browser_notify_set(s, "info");
    _browser_title_update();

    Evas_Object *win = evas_object_data_get(entry, "win");
    evas_object_del(win);
}


//
//
//
static void _new_group_bt_clicked_cb(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{

	Evas_Object *win, *bx, *hbx, *hbx2;
	Evas_Object *sp, *label, *ic;
	Evas_Object *bt;
	Evas_Object *entry;

    win = elm_win_util_standard_add("create_group", _("Create a new group"));
	elm_win_autodel_set(win, EINA_TRUE);
	evas_object_resize(win, 400, 400);
    evas_object_show(win);

	bx = elm_box_add(win);
	elm_win_resize_object_add(win, bx);
	evas_object_show(bx);

    hbx = elm_box_add(win);
    evas_object_size_hint_weight_set(hbx, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_box_horizontal_set(hbx, EINA_TRUE);
    elm_box_pack_end(bx, hbx);
    evas_object_show(hbx);

    label = elm_label_add(win);
    elm_object_text_set(label, _("Group name:"));
    evas_object_show(label);
    elm_box_pack_end(hbx, label);

    entry = elm_entry_add(win);
    evas_object_size_hint_weight_set(entry, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(entry, EVAS_HINT_FILL, EVAS_HINT_FILL);
    elm_entry_scrollbar_policy_set(entry, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_OFF);
    elm_entry_single_line_set(entry, EINA_TRUE);
    elm_entry_select_all(entry);
    elm_box_pack_end(hbx, entry);
    evas_object_show(entry);
    evas_object_data_set(entry, "win", win);

    sp = elm_separator_add(win);
    elm_separator_horizontal_set(sp, EINA_TRUE);
    evas_object_show(sp);
    elm_box_pack_end(bx, sp);

    hbx2 = elm_box_add(win);
    evas_object_size_hint_weight_set( hbx2, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_box_horizontal_set( hbx2, EINA_TRUE);
    elm_box_pack_end(bx,  hbx2);
    evas_object_show( hbx2);

	bt = elm_button_add(win);
	ic = elm_icon_add(win);
   	elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
   	elm_icon_standard_set(ic, "dialog-ok-apply");
   	elm_object_part_content_set(bt, "icon", ic);
	elm_object_text_set(bt, _("Ok"));
	elm_box_pack_end(hbx2, bt);
	evas_object_show(bt);
	evas_object_smart_callback_add(bt, "clicked", _add_group_bt_clicked_cb, entry);

	bt = elm_button_add(win);
	ic = elm_icon_add(win);
   	elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
   	elm_icon_standard_set(ic, "dialog-cancel");
   	elm_object_part_content_set(bt, "icon", ic);
	elm_object_text_set(bt, _("Cancel"));
	elm_box_pack_end(hbx2, bt);
	evas_object_show(bt);
	evas_object_smart_callback_add(bt, "clicked", window_clicked_close_cb, win);

    sp = elm_separator_add(win);
    elm_separator_horizontal_set(sp, EINA_TRUE);
    evas_object_show(sp);
    elm_box_pack_end(bx, sp);
}


//
//
//
static void _add_subgroup_bt_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{

	    Evas_Object *entry = data;
        const char *s;
        char t[128];

	    s = elm_object_text_get(entry);

        //Add new subgroup to groups&subgroups list.
		browser->group_selected->subgroups = eina_list_append(browser->group_selected->subgroups, strdup(s));

        //Add new subgroup to gengrid widget.
        Elm_Object_Item *it = elm_gengrid_item_append(browser->ggrid, browser->ggic, strdup(s), NULL, NULL);
        elm_gengrid_item_update(it);

        snprintf(t, sizeof(t), _("New %s subgroup has been added."), s);
        _browser_notify_set(t, "info");
        _browser_title_update();

        Evas_Object *win = evas_object_data_get(entry, "win");
        evas_object_del(win);
}


//
//
//
static void _new_subgroup_bt_clicked_cb(void *data  __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
    if(browser->group_selected)
    {
	    Evas_Object *win, *bx, *hbx, *hbx2;
	    Evas_Object *sp, *label, *ic;
	    Evas_Object *bt;
	    Evas_Object *entry;

        win = elm_win_util_standard_add("create_subgroup", _("Create a new subgroup to current group"));
	    elm_win_autodel_set(win, EINA_TRUE);
        evas_object_show(win);
	    evas_object_resize(win, 400, 400);
        elm_win_center(win, EINA_TRUE, EINA_TRUE);

	    bx = elm_box_add(win);
	    elm_win_resize_object_add(win, bx);
	    evas_object_show(bx);

        hbx = elm_box_add(win);
        evas_object_size_hint_weight_set(hbx, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        elm_box_horizontal_set(hbx, EINA_TRUE);
        elm_box_pack_end(bx, hbx);
        evas_object_show(hbx);

        label = elm_label_add(win);
        elm_object_text_set(label, _("SubGroup name:"));
        evas_object_show(label);
        elm_box_pack_end(hbx, label);

        entry = elm_entry_add(win);
        evas_object_size_hint_weight_set(entry, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_align_set(entry, EVAS_HINT_FILL, EVAS_HINT_FILL);
        elm_entry_scrollbar_policy_set(entry, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_OFF);
        elm_entry_single_line_set(entry, EINA_TRUE);
        elm_entry_select_all(entry);
        elm_box_pack_end(hbx, entry);
        evas_object_show(entry);
        evas_object_data_set(entry, "win", win);

        sp = elm_separator_add(win);
        elm_separator_horizontal_set(sp, EINA_TRUE);
        evas_object_show(sp);
        elm_box_pack_end(bx, sp);

        hbx2 = elm_box_add(win);
        evas_object_size_hint_weight_set( hbx2, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        elm_box_horizontal_set( hbx2, EINA_TRUE);
        elm_box_pack_end(bx,  hbx2);
        evas_object_show( hbx2);

    	bt = elm_button_add(win);
    	ic = elm_icon_add(win);
      	elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
   	    elm_icon_standard_set(ic, "dialog-ok");
      	elm_object_part_content_set(bt, "icon", ic);
    	elm_object_text_set(bt, _("Ok"));
    	elm_box_pack_end(hbx2, bt);
    	evas_object_show(bt);
    	evas_object_smart_callback_add(bt, "clicked", _add_subgroup_bt_clicked_cb, entry);

    	bt = elm_button_add(win);
    	ic = elm_icon_add(win);
     	elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
      	elm_icon_standard_set(ic, "dialog-cancel");
     	elm_object_part_content_set(bt, "icon", ic);
    	elm_object_text_set(bt, _("Cancel"));
	    elm_box_pack_end(hbx2, bt);
	    evas_object_show(bt);
	    evas_object_smart_callback_add(bt, "clicked", window_clicked_close_cb, win);

        sp = elm_separator_add(win);
        elm_separator_horizontal_set(sp, EINA_TRUE);
        evas_object_show(sp);
        elm_box_pack_end(bx, sp);
    }
    else
    {
        _browser_notify_set(_("You must select or create a group before creating a new subgroup."), "dialog-warning");
    }
}





static void  _delete_items_apply_bt_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
   char s[512];
    Eina_List *l, *selected;
    Elm_Object_Item *gg_it;
    Evas_Object *win = data;
    Item_Info *item;


    selected = (Eina_List*)elm_gengrid_selected_items_get(browser->ggrid);


    if(eina_list_count(selected) > 1)
        snprintf(s, sizeof(s), _("Items "));
    else
        snprintf(s, sizeof(s), _("Item "));
    
    EINA_LIST_FOREACH(selected, l, gg_it)
    {
        item = (Item_Info *)elm_object_item_data_get(gg_it);
        strcat(s, item->name);
        strcat(s, " ");

        item_remove(item);

        elm_object_item_del(gg_it);
    }

    if(eina_list_count(selected) > 1)
        strcat(s, _("have been removed."));
    else
        strcat(s, _("has been removed."));
    
    _browser_notify_set(s, "info");
    _browser_title_update();

    evas_object_del(win);
}

//
//Delete currently selected item.
//
static void _remove_ctxitem_clicked_cb(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
    char s[256];
    Eina_List *selected, *l;

    selected = (Eina_List*)elm_gengrid_selected_items_get(browser->ggrid);
    if (!selected) return;

    Evas_Object *win;
    Evas_Object *bx, *hbx;
    Evas_Object *bt;
    Evas_Object *ic, *label, *sp;
    Elm_Object_Item *gg_it;
    Item_Info *item;

    snprintf(s, sizeof(s), _("Are your sure you want to delete the following items:"));
    EINA_LIST_FOREACH(selected, l, gg_it)
    {
        item = (Item_Info *)elm_object_item_data_get(gg_it);
        strcat(s, item->name);
        strcat(s, " ");
    }
    strcat(s, "?");

    win = elm_win_util_standard_add("remove_items_dlg", _("Confirmation"));
    elm_win_autodel_set(win, EINA_TRUE);
    evas_object_show(win);
    evas_object_resize(win, 400, 400);
    elm_win_center(win, EINA_TRUE, EINA_TRUE);

    bx = elm_box_add(win);
    elm_win_resize_object_add(win, bx);
    evas_object_show(bx);

    sp = elm_separator_add(win);
    elm_separator_horizontal_set(sp, EINA_TRUE);
    evas_object_show(sp);
    elm_box_pack_end(bx, sp);

       hbx = elm_box_add(win);
       elm_box_horizontal_set(hbx, EINA_TRUE);
        elm_box_pack_end(bx, hbx);
        evas_object_size_hint_weight_set(hbx, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_show(hbx);

        label = elm_label_add(win);
        elm_object_text_set(label, s);
        elm_box_pack_end(hbx, label);
        evas_object_size_hint_weight_set(label, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_show(label);

        ic = elm_icon_add(win);
     	elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
      	elm_icon_standard_set(ic, "question-notify");
        elm_box_pack_end(hbx, ic);
        evas_object_size_hint_weight_set(ic, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_show(ic);

        sp = elm_separator_add(win);
        elm_separator_horizontal_set(sp, EINA_TRUE);
        elm_box_pack_end(bx, sp);
        evas_object_show(sp);

        hbx = elm_box_add(win);
        evas_object_size_hint_weight_set(hbx, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        elm_box_horizontal_set(hbx, EINA_TRUE);
        elm_box_pack_end(bx, hbx);
        evas_object_show(hbx);

    	bt = elm_button_add(win);
    	ic = elm_icon_add(win);
      	elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
   	    elm_icon_standard_set(ic, "dialog-ok-apply");
      	elm_object_part_content_set(bt, "icon", ic);
    	elm_object_text_set(bt, _("Ok"));
    	elm_box_pack_end(hbx, bt);
    	evas_object_show(bt);
    	evas_object_smart_callback_add(bt, "clicked", _delete_items_apply_bt_cb, win);

    	bt = elm_button_add(win);
    	ic = elm_icon_add(win);
     	elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
      	elm_icon_standard_set(ic, "dialog-cancel");
     	elm_object_part_content_set(bt, "icon", ic);
    	elm_object_text_set(bt, _("Cancel"));
	    elm_box_pack_end(hbx, bt);
	    evas_object_show(bt);
	    evas_object_smart_callback_add(bt, "clicked", window_clicked_close_cb, win);

        sp = elm_separator_add(win);
        elm_separator_horizontal_set(sp, EINA_TRUE);
        elm_box_pack_end(bx, sp);
        evas_object_show(sp);

    _browser_title_update(browser);
}



//
//Called when an item has been dragged to a new subgroup.
//In these case, we need to update its own subgroup.
static void _ggrid_moved_cb(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
{
    char *s;
    Item_Info *item;
    Elm_Object_Item *it;
    const Elm_Gengrid_Item_Class * gic;


    item = (Item_Info *)elm_object_item_data_get(event_info);

    //Try to found new subgroup from moved item after dropping it.
    it = event_info;
    do
    {
        it = elm_gengrid_item_prev_get(it);

        if(!it)
            break;

        gic = elm_gengrid_item_item_class_get(it);

        if(strcmp(gic->item_style, "group_index") == 0)
            break;

    } while(it);

    //If it is in subgroup ok, but sometimes it can be in noman's land.
    if(it)
    {
        s = (char *)elm_object_item_data_get(it);

        //Update subgroup item.
        item_subgroup_set(item, s);
        item_update(item);

        char t[128];
         snprintf(t, sizeof(t), _("Item %s has been moved to subgroup %s."), item->name, s);
        _browser_notify_set(t, "info");

        elm_object_item_data_set(event_info, item);
        elm_gengrid_item_update(event_info);
    }
    else
    {
        INF(_("Moved %s to unknown space."), item->name);
    }
}


//
//Select an item in gengrid and display informations about it.
//
static void _ggrid_selected_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
    if(data)
    {
        //Item_Info *item = NULL;
        Items_Browser *browser = (Items_Browser *) data;

        //item = (Item_Info *)elm_object_item_data_get(event_info);
/*
        snprintf(s, sizeof(s), _("\nFilename:%s</>"
                                "<br>Name:%s</>"
                                "<br>Locale:%s</>"
                                "<br>Group:%s</>"
		                           "<br>SubGroup:%s</>"
		                           "<br>Distractor 1 %s of type %s</>"
		                          "<br>Distractor 2 %s of type %s</>"
		                          "<br>Distractor 3 %s of type %s</>"
		                          "<br>Distractor 4 %s of type %s</>"
		                          "<br>Creation Date:%s</>"
                                "<br>Revision Date:%s</>"),
		                          ecore_file_file_get(item->filename),
		                          item->name,
		                          item->locale,
		                          item->group,
		                          item->subgroup,
		                          item_distractor_name_get(item, 0), item_distractor_type_get(item, 0),
		                          item_distractor_name_get(item, 1), item_distractor_type_get(item, 1),
		                          item_distractor_name_get(item, 2), item_distractor_type_get(item, 2),
		                          item_distractor_name_get(item, 3), item_distractor_type_get(item, 3),
		                          item->creation_date,
		                           item->revision_date);

*/

        elm_object_text_set(browser->item_desc_entry, "TEST");
    }
}



//
//Selected a group in hoversel and display all subgroups and items related to it.
//
static void _groups_hs_selected_cb(void *data __UNUSED__, Evas_Object *obj, void *event_info)
{
    Eina_List *items,*groups, *l;
    Elm_Object_Item *it = event_info;
    Group_Info *group;

    browser->group_selected = elm_object_item_data_get(it);
    elm_object_text_set(obj, elm_entry_markup_to_utf8(browser->group_selected->name));

    elm_gengrid_clear(browser->ggrid);

    groups = groups_list_get();
    items=items_list_get();
    l = NULL;

    EINA_LIST_FOREACH(groups, l, group)
    {
        if(strcmp(browser->group_selected->name, group->name) == 0)
        {
            Eina_List *l2;
            char *s;
        	EINA_LIST_FOREACH(group->subgroups, l2, s)
        	{
                elm_gengrid_item_append(browser->ggrid, browser->ggic, s, NULL, browser);

                Item_Info *item;
                Eina_List *l3;
                EINA_LIST_FOREACH(items, l3, item)
                {
                    if(strcmp(item->group, group->name) == 0 && strcmp(item->subgroup, s) == 0)
                    {
                        elm_gengrid_item_append(browser->ggrid, browser->gic, item, _ggrid_selected_cb, browser);
                    }
                }
            }
	    }
	}
	groups = groups_list_free(groups);
	items = items_list_free(items);
}


//
//Set subgroup's label in gengrid.
//
static char *_ggrid_subgroup_text_get(void *data, Evas_Object *obj __UNUSED__, const char *part __UNUSED__)
{
    const char *label = data;
    char s[256];

    snprintf(s, sizeof(s), _("Subgroup %s"), elm_entry_markup_to_utf8(label));
    return strdup(s);
}


//
//Set item's contents in gengrid.
//TODO:not used yet, can be used to add check/toggle....
/*
static Evas_Object *_ggrid_item_check_changed(void *data, Evas_Object *obj, const char *part)
{


}
*/

//
//Set item's label in gengrid.
//
static char *_ggrid_item_text_get(void *data, Evas_Object *obj __UNUSED__, const char *part __UNUSED__)
{
   Item_Info *item = data;

   return strdup(elm_entry_markup_to_utf8(item->name));
}


//
//Set item's contents in gengrid.
//TODO:check content not used yet.
static Evas_Object *_ggrid_content_get(void *data, Evas_Object *obj, const char *part)
{
   Item_Info *item = (Item_Info *)data;

   if (!strcmp(part, "elm.swallow.icon"))
     {
        Evas_Object *icon = elm_bg_add(obj);
        elm_bg_option_set(icon, ELM_BG_OPTION_SCALE);
        elm_bg_load_size_set(icon, 100, 100);
        elm_bg_file_set(icon, item->filename, "images/0");
       evas_object_size_hint_weight_set(icon, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	    evas_object_size_hint_aspect_set(icon, EVAS_ASPECT_CONTROL_BOTH, 1, 1);
    	evas_object_show(icon);
	    return icon;
     }/*
   else if (!strcmp(part, "elm.swallow.end"))
     {
        Evas_Object *ck;
        ck = elm_check_add(obj);
        evas_object_propagate_events_set(ck, 0);
        //elm_check_state_set(ck, it->onoff);
        //evas_object_smart_callback_add(ck, "changed", _grid_item_check_changed, data);
        evas_object_show(ck);
        return ck;
     }
     */
   return NULL;
}


//
//Append a new empty item.
//
static void _add_bt_clicked_cb(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
    if(browser->group_selected)
    {
        const char *s;
        Item_Info *item;
        Elm_Object_Item *it, *fit;

        item = item_new();
        item_group_set(item, browser->group_selected->name);
        s = item_write(item);
        item = item_free(item);
        item = item_read(s);
        fit = elm_gengrid_first_item_get(browser->ggrid);
        fit = elm_gengrid_item_next_get(fit);

        if(fit)
        {
           // it = elm_gengrid_item_sorted_insert(browser->ggrid, browser->gic, item, EINA_COMPARE_CB(strcoll), _ggrid_selected_cb, browser);
            it = elm_gengrid_item_insert_before(browser->ggrid, browser->gic, item, fit, _ggrid_selected_cb, browser);
        }
        else
        {
            it = elm_gengrid_item_append(browser->ggrid, browser->gic, item, _ggrid_selected_cb, browser);
        }
        elm_gengrid_item_update(it);
        elm_gengrid_item_selected_set(it, EINA_TRUE);

        char t[128];
         snprintf(t, sizeof(t), _("Item %s has been created."), item->name);
        _browser_notify_set(t, "info");

        _browser_title_update(browser);
    }
    else
    {
        _browser_notify_set(_("You must select or create a group before creating a new item."), "dialog-warning");
    }
}


//
//
//
static Eina_Bool _ggrid_state_get(void *data __UNUSED__, Evas_Object *obj __UNUSED__, const char *part __UNUSED__)
{
   return EINA_FALSE;
}


//
//Grid item deletion callback
//
static void _ggrid_del(void *data, Evas_Object *obj __UNUSED__)
{
    Item_Info *item = data;
    item = item_free(item);
}


//
//Set item's contents size in gengrid.
//
static void _size_slider_changed_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__)
{
   Evas_Object *grid = data;
   int size = elm_slider_value_get(obj);
   elm_gengrid_item_size_set(grid, size, size);

   setting_write("Item Browser/item size", size);
}


//
//Set gengrid horizontal mode.
//
/*
static void _horizontal_ck_changed_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__)
{
   Evas_Object *grid = data;
   elm_gengrid_horizontal_set(grid, elm_check_state_get(obj));
}
*/

//
//Update browser window title.
//
static void _browser_title_update()
{
    char s[128];
    Eina_List *items = NULL;
    Eina_List *groups = NULL;

    items = items_list_get(items);
    groups = groups_list_get(groups);

	snprintf(s, sizeof(s), _("Items Browser - %d items in %d groups"), eina_list_count(items), eina_list_count(groups));
	elm_win_title_set(browser->win, s);

    items = items_list_free(items);
    groups = groups_list_free(groups);
}


//
//Hide notifycation after set delay time.
//
static Eina_Bool _bubble_timer_cb(void *data __UNUSED__)
{
    evas_object_hide(browser->bubble);

    return EINA_FALSE;
}

//
//Show notifycation.
//
static void _browser_notify_set(const char *s, const char *ic __UNUSED__)
{
       	//elm_icon_standard_set(browser->bubble_ic, ic);
    	elm_object_text_set(browser->bubble_txt, s);
    	evas_object_show(browser->bubble);

        browser->bubble_timer = ecore_timer_add(5, _bubble_timer_cb, NULL);
}


//
//Update items descriptions.
//
static void  _item_description_apply_bt_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
    char s[256];
    Evas_Object *win = data;
    Evas_Object *entry[4];    
    Evas_Object *img[2];
    const char *f, *g;
    const char *f2, *g2;
    
	Elm_Object_Item *it = evas_object_data_get(win, "GenGrid Object Item");;
    Item_Info *item= (Item_Info*) elm_object_item_data_get(it);

    item_name_set(item, elm_object_text_get(evas_object_data_get(win, "Name Entry")));
    
    img[0] = evas_object_data_get(win, "Image Realistic");
    elm_image_file_get(img[0], &f, &g);
    //Don't try to update if isn't a new item image!
    if(f &&  (eina_str_has_extension(f, ".eet") == EINA_FALSE))
    {
        item_image_filename_set(item, f, 0);
    }
    img[1] = evas_object_data_get(win, "Image Abstract");
    elm_image_file_get(img[1], &f2, &g2);    
    //Don't try to update if isn't a new item image!
    if(f2 &&  (eina_str_has_extension(f2, ".eet") == EINA_FALSE))
    {
        item_image_filename_set(item, f2, 1);
    }
    elm_object_item_data_set(it, item);
    elm_gengrid_item_update(it);
    
    entry[0] = evas_object_data_get(win, "Graphemics Entry");
    entry[1] = evas_object_data_get(win, "Visuals Entry");
    entry[2] = evas_object_data_get(win, "Neutrals Entry");
    entry[3] = evas_object_data_get(win, "Semantics Entry");
        
    char **list;
    unsigned int i, num;
    list = eina_str_split_full(elm_object_text_get(entry[0]), ",", 0, &num);    
    if(list)
    {
        item_distractor_clear(item, GRAPHEMIC);
        
        for (i = 0; i < num; i++)
        {     
            if(strlen(list[i])>0)
            {
                item_distractor_add(item, list[i], GRAPHEMIC);
            }
        }
        free(list[0]);
        free(list);
	}	

    list = eina_str_split_full(elm_object_text_get(entry[1]), ",", 0, &num);    
    if(list)
    {
        item_distractor_clear(item, VISUAL);
        
        for (i = 0; i < num; i++)
        {     
            if(strlen(list[i])>0)
            {
                item_distractor_add(item, list[i], VISUAL);
            }
        }
        free(list[0]);
        free(list);
	}	
	
	
    list = eina_str_split_full(elm_object_text_get(entry[2]), ",", 0, &num);    
    if(list)
    {
        item_distractor_clear(item, NEUTRAL);
        
        for (i = 0; i < num; i++)
        {     
            if(strlen(list[i])>0)
            {
                item_distractor_add(item, list[i], NEUTRAL);
            }
        }
        free(list[0]);
        free(list);
	}	
	
    list = eina_str_split_full(elm_object_text_get(entry[3]), ",", 0, &num);    
    if(list)
    {
        item_distractor_clear(item, SEMANTIC);
        
        for (i = 0; i < num; i++)
        {     
            if(strlen(list[i])>0)
            {
                item_distractor_add(item, list[i], SEMANTIC);
            }
        }
        free(list[0]);
        free(list);
	}			



    item_update(item);

    snprintf(s, sizeof(s), _("Item %s has been updated."), item->name);
    _browser_notify_set(s, "info");

    evas_object_del(win);
}


//
//
//
static void  _action_bt_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
 	MyFileSelector *myfs = data;
    const char *sel;
    Evas_Object *img;

    img = evas_object_data_get(myfs->win, "image");
    sel = elm_fileselector_selected_get(myfs->fs);
    
    if(sel)
    {
    
        if((eina_str_has_extension(sel, ".png") == EINA_TRUE) ||
            (eina_str_has_extension(sel, "jpg") == EINA_TRUE) ||
            (eina_str_has_extension(sel, ".jpeg") == EINA_TRUE) ||    
            (eina_str_has_extension(sel, ".gif") == EINA_TRUE) ||                    
            (eina_str_has_extension(sel, ".bmp") == EINA_TRUE))
        elm_image_file_set(img, sel, NULL);
    }    
    myfileselector_close(myfs);
}


//
//
//
static void   _item_image_bt_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{    
    Evas_Object *img = data;
 	MyFileSelector *myfs;
 	
	myfs = myfileselector_add();
	myfileselector_set_title(myfs, "Select a file");
	evas_object_data_set(myfs->win, "image", img);
    evas_object_smart_callback_add(myfs->action_bt, "clicked", _action_bt_clicked_cb, myfs);
}



//
//
//
static void _description_ctxitem_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
	Evas_Object *win, *gd, *fr;
	Evas_Object *label, *ic, *img[2];
	Evas_Object *bt;
	Evas_Object *entry;
	Eina_List *l;
	char *s;

    if(!data)
        return;

	Elm_Object_Item *it = (Elm_Object_Item *)data;
    Item_Info *item = (Item_Info*) elm_object_item_data_get(it);

    win = elm_win_util_standard_add("item_description_dlg", _("Item Description"));
	elm_win_autodel_set(win, EINA_TRUE);
    elm_win_center(win, EINA_TRUE, EINA_TRUE);
	evas_object_data_set(win, "GenGrid Object Item", it);

    gd = elm_grid_add(win);
    elm_grid_size_set(gd, 100, 120);
    elm_win_resize_object_add(win, gd);
    evas_object_size_hint_weight_set(gd, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_show(gd);

    label = elm_label_add(win);
    elm_object_text_set(label, _("Item name:"));
    elm_grid_pack(gd, label, 0, 0, 50, 7);    
    evas_object_show(label);
    
    entry = elm_entry_add(win);
    elm_entry_scrollable_set(entry, EINA_TRUE);
    elm_entry_editable_set(entry, EINA_TRUE);
    elm_object_text_set(entry, item->name);
    elm_entry_single_line_set(entry, EINA_TRUE);
    elm_grid_pack(gd, entry, 40, 0, 50, 9);    
    evas_object_data_set(win, "Name Entry", entry);
    evas_object_show(entry);
    
    fr = elm_frame_add(win);
    elm_object_text_set(fr, _("Distractors"));
    elm_grid_pack(gd, fr, 0, 10, 100, 50);    
    evas_object_show(fr);
    
    label = elm_label_add(win);
    elm_object_text_set(label, _("Graphemics:"));
    elm_grid_pack(gd, label , 0, 17, 60, 8);
    evas_object_show(label);

    entry = elm_entry_add(win);
    elm_entry_scrollable_set(entry, EINA_TRUE);
    elm_entry_editable_set(entry, EINA_TRUE);
    
    	EINA_LIST_FOREACH(item->graphemics, l, s)
    	{
            elm_entry_entry_insert(entry, s);
            elm_entry_entry_insert(entry, ",");
        }
    elm_entry_single_line_set(entry, EINA_TRUE);
    elm_grid_pack(gd, entry, 40, 17, 60, 9);    
    evas_object_data_set(win, "Graphemics Entry", entry);
    evas_object_show(entry);
    
    label = elm_label_add(win);
    elm_object_text_set(label, _("Neutral:"));
    elm_grid_pack(gd, label , 0, 28, 60, 8);
    evas_object_show(label);

    entry = elm_entry_add(win);
    elm_entry_scrollable_set(entry, EINA_TRUE);
    elm_entry_editable_set(entry, EINA_TRUE);
    	EINA_LIST_FOREACH(item->neutrals, l, s)
    	{
            elm_entry_entry_insert(entry, s);
            elm_entry_entry_insert(entry, ",");
        }
    elm_entry_single_line_set(entry, EINA_TRUE);
    elm_grid_pack(gd, entry, 40, 28, 60, 9);    
    evas_object_data_set(win, "Neutrals Entry", entry);
    evas_object_show(entry);

    label = elm_label_add(win);
    elm_object_text_set(label, _("Semantics:"));
    elm_grid_pack(gd, label , 0, 39, 60, 8);
    evas_object_show(label);

    entry = elm_entry_add(win);
    elm_entry_scrollable_set(entry, EINA_TRUE);
    elm_entry_editable_set(entry, EINA_TRUE);
    	EINA_LIST_FOREACH(item->semantics, l, s)
    	{
            elm_entry_entry_insert(entry, s);
            elm_entry_entry_insert(entry, ",");
        }
    elm_entry_single_line_set(entry, EINA_TRUE);
    elm_grid_pack(gd, entry, 40, 39, 60, 9);    
    evas_object_data_set(win, "Semantics Entry", entry);
    evas_object_show(entry);
 
    label = elm_label_add(win);
    elm_object_text_set(label, _("Visuals:"));
    elm_grid_pack(gd, label , 0, 50, 60, 8);
    evas_object_show(label);

    entry = elm_entry_add(win);
    elm_entry_scrollable_set(entry, EINA_TRUE);
    elm_entry_editable_set(entry, EINA_TRUE);
    	EINA_LIST_FOREACH(item->visuals, l, s)
    	{
            elm_entry_entry_insert(entry, s);
            elm_entry_entry_insert(entry, ",");
        }
    elm_entry_single_line_set(entry, EINA_TRUE);
    elm_grid_pack(gd, entry, 40, 50, 60, 9);    
    evas_object_data_set(win, "Visuals Entry", entry);
    evas_object_show(entry);


    //
    //Image selectors.
    //
    fr = elm_frame_add(win);
    elm_object_text_set(fr, _("Realistic"));
    elm_grid_pack(gd, fr, 0, 64, 25, 25);    
    evas_object_show(fr);
    
    img[0] = elm_image_add(win);
    elm_image_smooth_set(img[0], EINA_TRUE);
    elm_image_aspect_fixed_set(img[0], EINA_TRUE);
    elm_image_resizable_set(img[0], EINA_TRUE, EINA_TRUE);
    elm_image_file_set(img[0], item->filename, "images/0");
    elm_grid_pack(gd, img[0], 6, 73, 12, 12);
    evas_object_show(img[0]);
    evas_object_data_set(win, "Image Realistic", img[0]);
	
	bt = elm_button_add(win);
    elm_object_text_set(bt, _("Realistic..."));
    ic = elm_icon_add(win);
    elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
    elm_icon_standard_set(ic, "document-open");
    elm_object_part_content_set(bt, "icon", ic);
    evas_object_smart_callback_add(bt, "clicked",  _item_image_bt_clicked_cb, img[0]);
    elm_grid_pack(gd, bt, 0, 90, 30, 10);
    evas_object_show(bt);
	
    fr = elm_frame_add(win);
    elm_object_text_set(fr, _("Abstract"));
    elm_grid_pack(gd, fr, 36, 64, 25, 25);
    evas_object_show(fr);
    
    img[1] = elm_image_add(win);
    elm_image_smooth_set(img[1], EINA_TRUE);
    elm_image_aspect_fixed_set(img[1], EINA_TRUE);
    elm_image_resizable_set(img[1], EINA_TRUE, EINA_TRUE);
    elm_image_file_set(img[1], item->filename, "images/1");
    elm_grid_pack(gd, img[1], 42, 73, 12, 12);    
    evas_object_show(img[1] );
    evas_object_data_set(win, "Image Abstract", img[1]);


	bt = elm_button_add(win);
    elm_object_text_set(bt, _("Abstract..."));
    ic = elm_icon_add(win);
    elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
    elm_icon_standard_set(ic, "document-open");
    elm_object_part_content_set(bt, "icon", ic);
    evas_object_smart_callback_add(bt, "clicked",  _item_image_bt_clicked_cb, img[1]);
    elm_grid_pack(gd, bt, 36, 90, 30, 10);
    evas_object_show(bt);
    //
    //End of image selectors.
    //
    
	bt = elm_button_add(win);
    evas_object_size_hint_weight_set(bt, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	ic = elm_icon_add(win);
   	elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
   	elm_icon_standard_set(ic, "dialog-ok-apply");
   	elm_object_part_content_set(bt, "icon", ic);
	elm_object_text_set(bt, _("Ok"));
    elm_grid_pack(gd, bt, 20, 110, 30, 10);    
	evas_object_show(bt);
	evas_object_smart_callback_add(bt, "clicked", _item_description_apply_bt_clicked_cb, win);

	bt = elm_button_add(win);
    evas_object_size_hint_weight_set(bt, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	ic = elm_icon_add(win);
   	elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
   	elm_icon_standard_set(ic, "dialog-cancel");
   	elm_object_part_content_set(bt, "icon", ic);
	elm_object_text_set(bt, _("Cancel"));
    elm_grid_pack(gd, bt, 60, 110, 30, 10);
	evas_object_show(bt);
	evas_object_smart_callback_add(bt, "clicked", window_clicked_close_cb, win);
	
	evas_object_resize(win, 410, 400);
	evas_object_show(win);
}



static void _export_action_bt_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
    char s[256];
    MyFileSelector *myfs = data;

    Item_Info *item = evas_object_data_get(myfs->win, "item");
    item_export(item, elm_fileselector_path_get(myfs->fs));

    snprintf(s, sizeof(s), _("Item %s has been exported."), item->name);
    _browser_notify_set(s, "info");
    _browser_title_update();

    myfileselector_close(myfs);
}


static void _export_ctxitem_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
 	MyFileSelector *myfs;
	Elm_Object_Item *it = (Elm_Object_Item *)data;
    Item_Info *item = (Item_Info*) elm_object_item_data_get(it);

	myfs = myfileselector_add();
	myfileselector_set_title(myfs, "Select a file");
    evas_object_data_set(myfs->win, "item", item);	
    elm_fileselector_folder_only_set(myfs->fs, EINA_TRUE);
    evas_object_smart_callback_add(myfs->action_bt, "clicked", _export_action_bt_clicked_cb, myfs);
}



//
//Display ctxpopup when double-click on an item's gengrid.
//
static void _ggrid_clickeddouble_cb(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
{
    Evas_Object *ctxpopup;
    Evas_Coord x,y;

    ctxpopup = elm_ctxpopup_add(obj);

    Elm_Object_Item *it = event_info;

    if(!it)
        return;
        
    //elm_gengrid_item_selected_set(it, EINA_TRUE);

    Evas_Object *ic = elm_icon_add(ctxpopup);
    elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
    elm_icon_standard_set(ic,  "properties-edit");
    elm_icon_resizable_set(ic, EINA_TRUE, EINA_TRUE);
    elm_ctxpopup_item_append(ctxpopup, _("Edit item..."), ic, _description_ctxitem_clicked_cb, it);

    ic = elm_icon_add(ctxpopup);
    elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
    elm_icon_standard_set(ic,  "export");
    elm_icon_resizable_set(ic, EINA_TRUE, EINA_TRUE);
    elm_ctxpopup_item_append(ctxpopup, _("Export item..."), ic, _export_ctxitem_clicked_cb, it);

    ic = elm_icon_add(ctxpopup);
    elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
    elm_icon_standard_set(ic,  "edit-trash");
    elm_icon_resizable_set(ic, EINA_TRUE, EINA_TRUE);
    elm_ctxpopup_item_append(ctxpopup, _("Delete selected items..."), ic, _remove_ctxitem_clicked_cb, it);

    //ctxpopup_item_new(ctxpopup, _("Set images..."), "properties-edit", event_info);
    //ctxpopup_item_new(ctxpopup, _("Move to group..."), "cut-edit", event_info);

    evas_pointer_canvas_xy_get(evas_object_evas_get(obj), &x, &y);
    evas_object_size_hint_max_set(ctxpopup, 240, 240);
    evas_object_move(ctxpopup, x, y);
    evas_object_show(ctxpopup);
}


//
//Create items browser.
//
void items_browser_new(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
    Evas_Object *bg, *bx, *hbx, *hbx2, *hbx3, *bt, *ic, *sl, *sp;
    Eina_List *groups;

	INF(_("Initialize Items Browser UI..."));

    browser = malloc(sizeof(*browser));
	browser->app = (App_Info *)data;
    browser->group_selected = NULL;
    int i;
    if((i = setting_int_read("Item Browser/item size")) == -1)
        i = 80;

	browser->win = elm_win_add(browser->app->win, "ItemsBrowser", ELM_WIN_BASIC);
	_browser_title_update();
	elm_win_autodel_set(browser->win, EINA_TRUE);
    elm_win_center(browser->win, EINA_TRUE, EINA_TRUE);
	evas_object_size_hint_min_set(browser->win, 740, 450);
	evas_object_size_hint_max_set(browser->win, 740, 450);

    bg = elm_bg_add(browser->win);
    evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_win_resize_object_add(browser->win, bg);
    evas_object_show(bg);

    bx = elm_box_add(browser->win);
    evas_object_size_hint_weight_set(bx, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_win_resize_object_add(browser->win, bx);
    evas_object_show(bx);

	//Setup bubble notify for user informations.
    browser->bubble = elm_bubble_add(browser->win);
	evas_object_color_set(browser->bubble, 0, 111, 255, 255);
    elm_bubble_pos_set(browser->bubble, ELM_BUBBLE_POS_TOP_LEFT);
    browser->bubble_txt = elm_label_add(browser->win);
    elm_object_content_set(browser->bubble, browser->bubble_txt);
    evas_object_color_set(browser->bubble_txt, 255, 255, 255, 255);
	//browser->bubble_ic = elm_icon_add(browser->win);
    //elm_icon_order_lookup_set(browser->bubble_ic, ELM_ICON_LOOKUP_FDO_THEME);    
    //elm_object_part_content_set(browser->bubble, "icon", browser->bubble_ic);
    evas_object_size_hint_weight_set(browser->bubble, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_resize(browser->bubble, 740, 100);
    evas_object_hide(browser->bubble);
    _browser_notify_set(_("Welcome to items browser!"), "info");
    
    browser->ggrid = elm_gengrid_add(browser->win);
	elm_gengrid_bounce_set(browser->ggrid, EINA_FALSE,  EINA_TRUE);
    elm_gengrid_item_size_set(browser->ggrid, i, i);
    elm_gengrid_horizontal_set(browser->ggrid, EINA_FALSE);
    elm_gengrid_multi_select_set(browser->ggrid, EINA_TRUE);
    elm_gengrid_select_mode_set(browser->ggrid, ELM_OBJECT_SELECT_MODE_ALWAYS);
    elm_gengrid_reorder_mode_set(browser->ggrid, EINA_TRUE);
    evas_object_size_hint_weight_set(browser->ggrid, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_min_set(browser->ggrid, 800, 600);
    elm_box_pack_end(bx, browser->ggrid);
    evas_object_show(browser->ggrid);
    evas_object_smart_callback_add(browser->ggrid, "moved", _ggrid_moved_cb, browser);
    evas_object_smart_callback_add(browser->ggrid, "clicked,double",_ggrid_clickeddouble_cb, NULL);

    //Navigation/filtering box.
    hbx = elm_box_add(browser->win);
    evas_object_size_hint_weight_set(hbx, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_box_horizontal_set(hbx, EINA_TRUE);
    elm_box_pack_end(bx, hbx);
    evas_object_show(hbx);

	browser->groups_hs = elm_hoversel_add(browser->win);
	elm_hoversel_hover_parent_set(browser->groups_hs, browser->win);
	elm_object_text_set(browser->groups_hs, _("Group selection"));
	evas_object_size_hint_weight_set(browser->groups_hs, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(browser->groups_hs, 0.5, 0.5);
	elm_box_pack_end(hbx, browser->groups_hs);
	evas_object_show(browser->groups_hs);

    //Fill hoversel group selection with all item's groups.
	groups = groups_list_get();
    Eina_List *l;
    Group_Info *group;
	EINA_LIST_FOREACH(groups, l, group)
	{
		elm_hoversel_item_add(browser->groups_hs, elm_entry_markup_to_utf8(group->name), NULL, ELM_ICON_NONE, NULL, group);
	}
    evas_object_smart_callback_add(browser->groups_hs, "selected", _groups_hs_selected_cb, NULL);
    groups = groups_list_free(groups);


    sl = elm_slider_add(browser->win);
    elm_slider_indicator_show_set(sl, EINA_FALSE);
    elm_object_text_set(sl, _("Item size"));
    evas_object_size_hint_weight_set(sl, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_slider_span_size_set(sl, 100);
    elm_slider_min_max_set(sl, 50, 400);
    evas_object_smart_callback_add(sl, "changed", _size_slider_changed_cb, browser->ggrid);
    elm_box_pack_end(hbx, sl);
    evas_object_show(sl);
    elm_slider_value_set(sl, i);
/*
    ck = elm_check_add(browser->win);
    elm_object_text_set(ck, _("Horizontal"));
    evas_object_smart_callback_add(ck, "changed", _horizontal_ck_changed_cb, browser->ggrid);
    elm_box_pack_end(hbx, ck);
    evas_object_show(ck);
*/
    sp = elm_separator_add(browser->win);
    elm_separator_horizontal_set(sp, EINA_TRUE);
    elm_box_pack_end(bx, sp);
    evas_object_show(sp);

    //Items informations/settings.
    hbx2 = elm_box_add(browser->win);
    evas_object_size_hint_weight_set(hbx2, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_box_horizontal_set(hbx2, EINA_TRUE);
    elm_box_pack_end(bx, hbx2);
    evas_object_show(hbx2);

    browser->item_desc_entry = elm_entry_add(browser->win);
    elm_entry_scrollable_set(browser->item_desc_entry, EINA_TRUE);
   	//elm_entry_line_wrap_set(browser->item_desc_entry, ELM_WRAP_MIXED);
    //elm_entry_single_line_set(browser->item_desc_entry, EINA_FALSE);
  	elm_entry_editable_set(browser->item_desc_entry, EINA_FALSE);
    elm_entry_select_all(browser->item_desc_entry);
    elm_box_pack_end(hbx2, browser->item_desc_entry);
    evas_object_size_hint_weight_set(browser->item_desc_entry, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(browser->item_desc_entry, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_show(browser->item_desc_entry);

    sp = elm_separator_add(browser->win);
    elm_separator_horizontal_set(sp, EINA_TRUE);
    elm_box_pack_end(bx, sp);
    evas_object_show(sp);

    //Actions box.
    hbx3 = elm_box_add(browser->win);
    evas_object_size_hint_weight_set(hbx3, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_box_horizontal_set(hbx3, EINA_TRUE);
    elm_box_pack_end(bx, hbx3);
    evas_object_show(hbx3);

    bt = elm_button_add(browser->win);
    elm_object_text_set(bt, _("Add group"));
    ic = elm_icon_add(browser->win);
    elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
    elm_icon_standard_set(ic, "add-edit");
    elm_object_part_content_set(bt, "icon", ic);
    elm_box_pack_end(hbx3, bt);
    evas_object_show(bt);
    evas_object_smart_callback_add(bt, "clicked", _new_group_bt_clicked_cb, NULL);

    bt = elm_button_add(browser->win);
    elm_object_text_set(bt, _("Add subgroup"));
    ic = elm_icon_add(browser->win);
    elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
    elm_icon_standard_set(ic, "add-edit");
    elm_object_part_content_set(bt, "icon", ic);
    elm_box_pack_end(hbx3, bt);
    evas_object_show(bt);
    evas_object_smart_callback_add(bt, "clicked", _new_subgroup_bt_clicked_cb, NULL);

    bt = elm_button_add(browser->win);
    elm_object_text_set(bt, _("Add item"));
    ic = elm_icon_add(browser->win);
    elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
    elm_icon_standard_set(ic, "add-edit");
    elm_object_part_content_set(bt, "icon", ic);
    elm_box_pack_end(hbx3, bt);
    evas_object_show(bt);
    evas_object_smart_callback_add(bt, "clicked", _add_bt_clicked_cb, NULL);

    bt = elm_button_add(browser->win);
    elm_object_text_set(bt, _("Import item"));
    ic = elm_icon_add(browser->win);
    elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
    elm_icon_standard_set(ic, "import");
    elm_object_part_content_set(bt, "icon", ic);
    evas_object_smart_callback_add(bt, "clicked", _import_bt_clicked_cb, NULL);
    elm_box_pack_end(hbx3, bt);
    evas_object_show(bt);

    bt = elm_button_add(browser->win);
    elm_object_text_set(bt, _("Close"));
	ic = elm_icon_add(browser->win);
   	elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
	elm_icon_standard_set(ic, "window-close");
   	elm_object_part_content_set(bt, "icon", ic);
    elm_box_pack_end(hbx3, bt);
    evas_object_show(bt);
    evas_object_smart_callback_add(bt, "clicked", _close_bt_clicked_cb, NULL);

    sp = elm_separator_add(browser->win);
    elm_separator_horizontal_set(sp, EINA_TRUE);
    evas_object_show(sp);
    elm_box_pack_end(bx, sp);

    browser->gic = elm_gengrid_item_class_new();
    browser->gic->item_style = "default";
    browser->gic->func.text_get = _ggrid_item_text_get;
    browser->gic->func.content_get = _ggrid_content_get;
    browser->gic->func.state_get = _ggrid_state_get;
    browser->gic->func.del = _ggrid_del;

    browser->ggic = elm_gengrid_item_class_new();
    browser->ggic->item_style = "group_index";
    browser->ggic->func.text_get = _ggrid_subgroup_text_get;
    browser->ggic->func.content_get = NULL;
    browser->ggic->func.state_get = NULL;
    browser->ggic->func.del = NULL;

    evas_object_resize(browser->win, 600, 600);
    evas_object_show(browser->win);
  	elm_win_center(browser->win, EINA_TRUE, EINA_TRUE);
}
