/*
 * patients_browser.c
 * This file is part of FriLogos
 *
 * Copyright (C) 2012 - Alexandre Dussart
 *
 * FriLogos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * FriLogos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FriLogos. If not, see <http://www.gnu.org/licenses/>.
 */


#include "frilogos.h"


typedef struct
{
	Evas_Object *win;

    //Notification widgets.
	Evas_Object *bubble;
	Evas_Object *bubble_txt;;
	Evas_Object *bubble_ic;

    //Browser Widgets.
	Evas_Object *ggrid;
    Evas_Object *patient_desc_entry;

    Elm_Gengrid_Item_Class *gic;

    Patient_Info *patient_selected;

	App_Info *app;
} Patients_Browser;

Patients_Browser *browser;


//Browser local functions.
static void _browser_title_update();
static void _browser_notify_set(const char *s, const char *ic);

//Widgets callbacks.
static void _close_bt_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__);
static void _add_bt_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__);
static void _description_ctxitem_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__);
static void _remove_ctxitem_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__);
//static void _size_sl_changed_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__);

//GenGrid specific callbacks.
static void _ggrid_selected_cb(void *data, Evas_Object *obj, void *event_info);
static void  _ggrid_del(void *data, Evas_Object *obj __UNUSED__);
static char * _ggrid_text_get(void *data, Evas_Object *obj __UNUSED__, const char *part __UNUSED__);
static Evas_Object* _ggrid_content_get(void *data, Evas_Object *obj, const char *part);
static Eina_Bool _ggrid_state_get(void *data __UNUSED__, Evas_Object *obj __UNUSED__, const char *part __UNUSED__);
static void _ggrid_clickeddouble_cb(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info);


//
//Close browser and freeing some allocated stuff.
//
static void _close_bt_clicked_cb(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
    browser->patient_selected = patient_free(browser->patient_selected);
	evas_object_del(browser->win);
	free(browser);
}




static void  _delete_patients_apply_bt_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
    char s[512];
    Eina_List *l, *selected;
    Elm_Object_Item *gg_it;
    Evas_Object *win = data;
    Patient_Info *patient;


    selected = (Eina_List*)elm_gengrid_selected_items_get(browser->ggrid);

    snprintf(s, sizeof(s), _("Following patients files:"));
    EINA_LIST_FOREACH(selected, l, gg_it)
    {
        patient = (Patient_Info *)elm_object_item_data_get(gg_it);
        strcat(s, patient->firstname);
        strcat(s, " ");
        strcat(s, patient->lastname);
        strcat(s, "  ");

        patient_remove(patient);

        elm_object_item_del(gg_it);
    }

    strcat(s,  _("have been removed."));
    _browser_notify_set(s, "info-notify");
    _browser_title_update();

    evas_object_del(win);
}

//
//Delete currently selected patient.
//
static void _remove_ctxitem_clicked_cb(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
    char s[512];
    Eina_List *selected, *l;

    selected = (Eina_List*)elm_gengrid_selected_items_get(browser->ggrid);
    if (!selected) return;

    Evas_Object *win;
    Evas_Object *bx, *hbx;
    Evas_Object *bt;
    Evas_Object *ic, *label, *sp;
    Elm_Object_Item *gg_it;
    Patient_Info *patient;

    snprintf(s, sizeof(s), _("Are your sure you want to delete the following patients files:"));
    EINA_LIST_FOREACH(selected, l, gg_it)
    {
        patient = (Patient_Info *)elm_object_item_data_get(gg_it);
        strcat(s, patient->firstname);
        strcat(s, " ");
    }
    strcat(s, "?");

    win = elm_win_util_standard_add("remove_patients_dlg", _("Confirmation"));
    elm_win_autodel_set(win, EINA_TRUE);
    evas_object_show(win);
    evas_object_resize(win, 400, 400);
    elm_win_center(win, EINA_TRUE, EINA_TRUE);

   bx = elm_box_add(win);
    elm_win_resize_object_add(win, bx);
    evas_object_show(bx);

       sp = elm_separator_add(win);
     elm_separator_horizontal_set(sp, EINA_TRUE);
       evas_object_show(sp);
       elm_box_pack_end(bx, sp);

       hbx = elm_box_add(win);
       elm_box_horizontal_set(hbx, EINA_TRUE);
        elm_box_pack_end(bx, hbx);
        evas_object_size_hint_weight_set(hbx, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_show(hbx);

        label = elm_label_add(win);
        elm_object_text_set(label, s);
       elm_win_resize_object_add(win, label);
        elm_box_pack_end(hbx, label);
        evas_object_size_hint_weight_set(label, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_show(label);

        ic = elm_icon_add(win);
     	elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
      	elm_icon_standard_set(ic, "question-notify");
        elm_win_resize_object_add(ic, label);
        elm_box_pack_end(hbx, ic);
        evas_object_size_hint_weight_set(ic, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_show(ic);

        sp = elm_separator_add(win);
        elm_separator_horizontal_set(sp, EINA_TRUE);
        elm_box_pack_end(bx, sp);
        evas_object_show(sp);

        hbx = elm_box_add(win);
        evas_object_size_hint_weight_set(hbx, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        elm_box_horizontal_set(hbx, EINA_TRUE);
        elm_box_pack_end(bx, hbx);
        evas_object_show(hbx);

    	bt = elm_button_add(win);
    	ic = elm_icon_add(win);
      	elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
   	    elm_icon_standard_set(ic, "apply-window");
      	elm_object_part_content_set(bt, "icon", ic);
    	elm_object_text_set(bt, _("Ok"));
    	elm_box_pack_end(hbx, bt);
    	evas_object_show(bt);
    	evas_object_smart_callback_add(bt, "clicked", _delete_patients_apply_bt_cb, win);

    	bt = elm_button_add(win);
    	ic = elm_icon_add(win);
     	elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
      	elm_icon_standard_set(ic, "window-close");
     	elm_object_part_content_set(bt, "icon", ic);
    	elm_object_text_set(bt, _("Close"));
	    elm_box_pack_end(hbx, bt);
	    evas_object_show(bt);
	    evas_object_smart_callback_add(bt, "clicked", window_clicked_close_cb, win);

        sp = elm_separator_add(win);
        elm_separator_horizontal_set(sp, EINA_TRUE);
        elm_box_pack_end(bx, sp);
        evas_object_show(sp);

    _browser_title_update(browser);
}


//
//
//Select a patient in gengrid and display informations about it.
//
static void _ggrid_selected_cb(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
/*
    if(data)
    {
        //Patient_Info *patient = NULL;

        //patient = (Patient_Info *)elm_object_item_data_get(event_info);

        snprintf(s, sizeof(s), _("\nFilename:%s</>"
                                "<br>Name:%s</>"
                                "<br>Locale:%s</>"
                                "<br>Group:%s</>"
		                           "<br>SubGroup:%s</>"
		                           "<br>Distractor 1 %s of type %s</>"
		                          "<br>Distractor 2 %s of type %s</>"
		                          "<br>Distractor 3 %s of type %s</>"
		                          "<br>Distractor 4 %s of type %s</>"
		                          "<br>Creation Date:%s</>"
                                "<br>Revision Date:%s</>"),
		                          ecore_file_file_get(patient->filename),
		                          patient->firstname,
		                          patient->lastname,
		                          patient->birthday,
		                          patient->creation_date,
		                           patient->revision_date);
        elm_object_text_set(browser->patient_desc_entry, "TEST");
    }
    */
}



//
//Set patient's label in gengrid.
//
static char *_ggrid_text_get(void *data, Evas_Object *obj __UNUSED__, const char *part __UNUSED__)
{
    char s[128];
    Patient_Info *patient = (Patient_Info *)data;

    snprintf(s, sizeof(s), "%s %s", elm_entry_markup_to_utf8(patient->firstname), elm_entry_markup_to_utf8( patient->lastname));
    return strdup(s);
}


//
//Set patient's contents in gengrid.
//TODO:check content not used yet.
static Evas_Object *_ggrid_content_get(void *data __UNUSED__, Evas_Object *obj, const char *part)
{
   if (!strcmp(part, "elm.swallow.icon"))
     {
        Evas_Object *icon = elm_bg_add(obj);
        elm_bg_option_set(icon, ELM_BG_OPTION_SCALE);
        elm_bg_file_set(icon, frilogos_edje_theme_file_get(), "user/male");
	    evas_object_size_hint_aspect_set(icon, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);
    	evas_object_show(icon);
	    return icon;
     }

   return NULL;
}


//
//Append a new empty patient file.
//
static void _add_bt_clicked_cb(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
    const char *s;
    char t[128];
    Patient_Info *patient;
    Elm_Object_Item *it;

    patient = patient_new();
    s = patient_write(patient);
    patient = patient_free(patient);
    patient = patient_read(s);

    it = elm_gengrid_item_append(browser->ggrid, browser->gic, patient, _ggrid_selected_cb, browser);
    elm_gengrid_item_update(it);
    elm_gengrid_item_selected_set(it, EINA_TRUE);

    snprintf(t, sizeof(t), _("Patient %s %s has been created."), patient->firstname, patient->lastname);
    _browser_notify_set(t, "info-notify");

    _browser_title_update(browser);
}


//
//
//
static Eina_Bool _ggrid_state_get(void *data __UNUSED__, Evas_Object *obj __UNUSED__, const char *part __UNUSED__)
{
   return EINA_FALSE;
}


//
//Grid patient deletion callback
//
static void _ggrid_del(void *data, Evas_Object *obj __UNUSED__)
{

    Patient_Info *patient = data;
    patient = patient_free(patient);
}


//
//Update browser window title.
//
static void _browser_title_update()
{
    char s[128];
    Eina_List *patients = NULL;

    patients = patients_list_get();

    if(patients)
	    snprintf(s, sizeof(s), _("Patients Browser - %d patients"), eina_list_count(patients));
    else
	    snprintf(s, sizeof(s), _("Patients Browser - 0 patient"));
	elm_win_title_set(browser->win, s);

    patients = patients_list_free(patients);
}


//
//Hide notifycation after set delay time.
//
static Eina_Bool _bubble_timer_cb(void *data __UNUSED__)
{
    evas_object_hide(browser->bubble);

    return EINA_FALSE;
}


//
//Show notifycation.
//
static void _browser_notify_set(const char *s, const char *ic)
{
       	elm_icon_standard_set(browser->bubble_ic, ic);
    	elm_object_text_set(browser->bubble_txt, s);
    	evas_object_show(browser->bubble);

        ecore_timer_add(4, _bubble_timer_cb, NULL);
}


//
//Update current selected patient informations.
//
static void  _patient_description_apply_bt_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
    Evas_Object *win = data;

	Elm_Object_Item *it = evas_object_data_get(win, "GenGrid Object Item");;
    Patient_Info *patient= (Patient_Info*) elm_object_item_data_get(it);

    patient_firstname_set(patient, elm_object_text_get(evas_object_data_get(win, "First Name Entry")));
    patient_lastname_set(patient, elm_object_text_get(evas_object_data_get(win, "Last Name Entry")));
    patient_birthday_set(patient, elm_object_text_get(evas_object_data_get(win, "BirthDay Entry")));
    patient_update(patient);

    elm_object_item_data_set(it, patient);
    elm_gengrid_item_update(it);

    evas_object_del(win);
}


static void _description_ctxitem_clicked_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
	Evas_Object *win, *bx, *hbx, *hbx2;
	Evas_Object *sp, *label, *ic;
	Evas_Object *bt;
	Evas_Object *entry;

    if(!data)
        return;

	Elm_Object_Item *it = (Elm_Object_Item *)data;
    Patient_Info *patient = (Patient_Info*) elm_object_item_data_get(it);

    INF(patient->firstname);

    win = elm_win_util_standard_add("patient_description_dlg", _("Patient Description"));
	elm_win_autodel_set(win, EINA_TRUE);
    evas_object_show(win);
	evas_object_resize(win, 400, 400);
    elm_win_center(win, EINA_TRUE, EINA_TRUE);
	evas_object_data_set(win, "GenGrid Object Item", it);

	bx = elm_box_add(win);
	elm_win_resize_object_add(win, bx);
	evas_object_show(bx);

    sp = elm_separator_add(win);
    elm_separator_horizontal_set(sp, EINA_TRUE);
    evas_object_show(sp);
    elm_box_pack_end(bx, sp);

    hbx = elm_box_add(win);
    evas_object_size_hint_weight_set(hbx, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_box_horizontal_set(hbx, EINA_TRUE);
    elm_box_pack_end(bx, hbx);
    evas_object_show(hbx);

    label = elm_label_add(win);
    elm_object_text_set(label, _("First Name:"));
    evas_object_show(label);
    elm_box_pack_end(hbx, label);

    entry = elm_entry_add(win);
    elm_object_text_set(entry, patient_firstname_get(patient));
    evas_object_size_hint_weight_set(entry, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(entry, EVAS_HINT_FILL, EVAS_HINT_FILL);
    elm_entry_scrollbar_policy_set(entry, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_OFF);
    elm_entry_single_line_set(entry, EINA_TRUE);
    elm_entry_select_all(entry);
    elm_box_pack_end(hbx, entry);
    evas_object_show(entry);
    evas_object_data_set(win, "First Name Entry", entry);

    sp = elm_separator_add(win);
    elm_separator_horizontal_set(sp, EINA_TRUE);
    evas_object_show(sp);
    elm_box_pack_end(bx, sp);

    hbx = elm_box_add(win);
    evas_object_size_hint_weight_set(hbx, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_box_horizontal_set(hbx, EINA_TRUE);
    elm_box_pack_end(bx, hbx);
    evas_object_show(hbx);

    label = elm_label_add(win);
    elm_object_text_set(label, _("Last Name:"));
    evas_object_show(label);
    elm_box_pack_end(hbx, label);

    entry = elm_entry_add(win);
    elm_object_text_set(entry, patient_lastname_get(patient));
    evas_object_size_hint_weight_set(entry, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(entry, EVAS_HINT_FILL, EVAS_HINT_FILL);
    elm_entry_scrollbar_policy_set(entry, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_OFF);
    elm_entry_single_line_set(entry, EINA_TRUE);
    elm_entry_select_all(entry);
    elm_box_pack_end(hbx, entry);
    evas_object_show(entry);
    evas_object_data_set(win, "Last Name Entry", entry);

    sp = elm_separator_add(win);
    elm_separator_horizontal_set(sp, EINA_TRUE);
    evas_object_show(sp);
    elm_box_pack_end(bx, sp);

    hbx2 = elm_box_add(win);
    evas_object_size_hint_weight_set(hbx2, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_box_horizontal_set( hbx2, EINA_TRUE);
    elm_box_pack_end(bx,  hbx2);
    evas_object_show( hbx2);

	bt = elm_button_add(win);
    evas_object_size_hint_weight_set(bt, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	ic = elm_icon_add(win);
   	elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
   	elm_icon_standard_set(ic, "apply-window");
   	elm_object_part_content_set(bt, "icon", ic);
	elm_object_text_set(bt, _("Ok"));
	elm_box_pack_end(hbx2, bt);
	evas_object_show(bt);
	evas_object_smart_callback_add(bt, "clicked", _patient_description_apply_bt_clicked_cb, win);

	bt = elm_button_add(win);
    evas_object_size_hint_weight_set(bt, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	ic = elm_icon_add(win);
   	elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
   	elm_icon_standard_set(ic, "window-close");
   	elm_object_part_content_set(bt, "icon", ic);
	elm_object_text_set(bt, _("Close"));
	elm_box_pack_end(hbx2, bt);
	evas_object_show(bt);
	evas_object_smart_callback_add(bt, "clicked", window_clicked_close_cb, win);

    sp = elm_separator_add(win);
    elm_separator_horizontal_set(sp, EINA_TRUE);
    evas_object_show(sp);
    elm_box_pack_end(bx, sp);
}




//
//Display ctxpopup when double-click on an patient's gengrid.
//
static void _ggrid_clickeddouble_cb(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
{
    Evas_Object *ctxpopup;
    Evas_Coord x,y;

    ctxpopup = elm_ctxpopup_add(obj);

    Elm_Object_Item *it = event_info;

    if(!it)
        return;

    Evas_Object *ic = elm_icon_add(ctxpopup);
    elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
    elm_icon_standard_set(ic,  "properties-edit");
    elm_icon_resizable_set(ic, EINA_TRUE, EINA_TRUE);
    elm_ctxpopup_item_append(ctxpopup, _("Edit description..."), ic, _description_ctxitem_clicked_cb, it);

    ic = elm_icon_add(ctxpopup);
    elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
    elm_icon_standard_set(ic,  "delete-edit");
    elm_icon_resizable_set(ic, EINA_TRUE, EINA_TRUE);
    elm_ctxpopup_item_append(ctxpopup, _("Delete selected items..."), ic, _remove_ctxitem_clicked_cb, it);

    evas_pointer_canvas_xy_get(evas_object_evas_get(obj), &x, &y);
    evas_object_size_hint_max_set(ctxpopup, 240, 240);
    evas_object_move(ctxpopup, x, y);
    evas_object_show(ctxpopup);
}


//
//Create patients browser.
//
void patients_browser_new(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{

    Evas_Object *bg, *bx, *hbx2, *hbx3, *bt, *ic, *sp;

	INF(_("Initialize Patients Browser UI..."));

    browser = malloc(sizeof(*browser));
	browser->app = (App_Info *)data;
    browser->patient_selected = NULL;

	browser->win = elm_win_add(browser->app->win, "PatientsBrowser", ELM_WIN_BASIC);
	_browser_title_update();
	elm_win_autodel_set(browser->win, EINA_TRUE);
	evas_object_size_hint_min_set(browser->win, 740, 450);
	evas_object_size_hint_max_set(browser->win, 740, 450);

    bg = elm_bg_add(browser->win);
    evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_win_resize_object_add(browser->win, bg);
    evas_object_show(bg);

    bx = elm_box_add(browser->win);
    evas_object_size_hint_weight_set(bx, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_win_resize_object_add(browser->win, bx);
    evas_object_show(bx);


	//Setup bubble notify for user informations.
    browser->bubble = elm_bubble_add(browser->win);
    browser->bubble_txt = elm_label_add(browser->win);
    elm_object_text_set(browser->bubble_txt, _("Welcome to patients browser!"));
    elm_object_content_set(browser->bubble, browser->bubble_txt);
	browser->bubble_ic = elm_icon_add(browser->win);
   	elm_icon_order_lookup_set(browser->bubble_ic, ELM_ICON_LOOKUP_FDO_THEME);    
    elm_object_part_content_set(browser->bubble, "icon", browser->bubble_ic);
    elm_box_pack_end(bx, browser->bubble);    
    evas_object_size_hint_weight_set(browser->bubble, EVAS_HINT_EXPAND, 0.0);
    evas_object_size_hint_align_set(browser->bubble, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_hide(browser->bubble);


    browser->ggrid = elm_gengrid_add(browser->win);
    elm_gengrid_item_size_set(browser->ggrid, 180, 100);
    elm_gengrid_horizontal_set(browser->ggrid, EINA_FALSE);
    elm_gengrid_multi_select_set(browser->ggrid, EINA_TRUE);
    elm_gengrid_select_mode_set(browser->ggrid, ELM_OBJECT_SELECT_MODE_ALWAYS);
    elm_gengrid_reorder_mode_set(browser->ggrid, EINA_FALSE);
    evas_object_size_hint_weight_set(browser->ggrid, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_min_set(browser->ggrid, 600, 500);
    elm_box_pack_end(bx, browser->ggrid);
    evas_object_show(browser->ggrid);
    evas_object_smart_callback_add(browser->ggrid, "clicked,double",_ggrid_clickeddouble_cb, NULL);


    //Patient informations/settings.
    hbx2 = elm_box_add(browser->win);
    evas_object_size_hint_weight_set(hbx2, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_box_horizontal_set(hbx2, EINA_TRUE);
    elm_box_pack_end(bx, hbx2);
    evas_object_show(hbx2);

    browser->patient_desc_entry = elm_entry_add(browser->win);
    elm_entry_scrollable_set(browser->patient_desc_entry, EINA_TRUE);
  	elm_entry_editable_set(browser->patient_desc_entry, EINA_FALSE);
    elm_entry_select_all(browser->patient_desc_entry);
    elm_box_pack_end(hbx2, browser->patient_desc_entry);
    evas_object_size_hint_weight_set(browser->patient_desc_entry, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(browser->patient_desc_entry, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_show(browser->patient_desc_entry);

    sp = elm_separator_add(browser->win);
    elm_separator_horizontal_set(sp, EINA_TRUE);
    elm_box_pack_end(bx, sp);
    evas_object_show(sp);

    //Actions box.
    hbx3 = elm_box_add(browser->win);
    evas_object_size_hint_weight_set(hbx3, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_box_horizontal_set(hbx3, EINA_TRUE);
    elm_box_pack_end(bx, hbx3);
    evas_object_show(hbx3);

    bt = elm_button_add(browser->win);
    elm_object_text_set(bt, _("Add patient"));
    ic = elm_icon_add(browser->win);
    elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
    elm_icon_standard_set(ic, "add-edit");
    elm_object_part_content_set(bt, "icon", ic);
    elm_box_pack_end(hbx3, bt);
    evas_object_show(bt);
    evas_object_smart_callback_add(bt, "clicked", _add_bt_clicked_cb, browser);

    bt = elm_button_add(browser->win);
    elm_object_text_set(bt, _("Close"));
	ic = elm_icon_add(browser->win);
   	elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
	elm_icon_standard_set(ic, "window-close");
   	elm_object_part_content_set(bt, "icon", ic);
    elm_box_pack_end(hbx3, bt);
    evas_object_show(bt);
    evas_object_smart_callback_add(bt, "clicked", _close_bt_clicked_cb, browser->ggrid);

    sp = elm_separator_add(browser->win);
    elm_separator_horizontal_set(sp, EINA_TRUE);
    evas_object_show(sp);
    elm_box_pack_end(bx, sp);

    browser->gic = elm_gengrid_item_class_new();
    browser->gic->item_style = "default";
    browser->gic->func.text_get = _ggrid_text_get;
    browser->gic->func.content_get = _ggrid_content_get;
    browser->gic->func.state_get = _ggrid_state_get;
    browser->gic->func.del = _ggrid_del;

    Eina_List *patients, *l;
    Patient_Info *patient;
    patients =patients_list_get();
    l = NULL;

    EINA_LIST_FOREACH(patients, l, patient)
    {
        INF(patient->filename);
        elm_gengrid_item_append(browser->ggrid, browser->gic, patient, _ggrid_selected_cb, patient);
	}

	patients = patients_list_free(patients);

    elm_gengrid_item_class_free(browser->gic);

    evas_object_resize(browser->win, 600, 600);
    evas_object_show(browser->win);
  	elm_win_center(browser->win, EINA_TRUE, EINA_TRUE);
}
