/*
 * halloc.h
 * This file is part of FriLogos
 *
 * Modified Copyright (C) 2012 - Alexandre Dussart
 * Derivated from HAlloc project HaypoCALC by Haypo (http://www.haypocalc.com/).
 *
 * FriLogos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * FriLogos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FriLogos. If not, see <http://www.gnu.org/licenses/>.
 */



#ifndef HAlloc_H
#define HAlloc_H
//-----------------------------------------------------------------------------
#include <libfrilogos.h>
//-----------------------------------------------------------------------------

// Allocation scurise : vrifie que la mmoire est effectivement
// alloue lors des allocations
#define DEBUG_MALLOC

// Conteneur Malloc :
// - Liste les pointeurs allous (vite les fuites de mmoire)
// - Avant de librer un pointeur, vrifie qu'il est effectivement allou!
#define DEBUG_MALLOC_CONTENEUR

// Tampon anti-overflow :
// - Vrifie que la mmoire alloue n'ai pas t modifi en dehors
//   de l'espace allou
#define DEBUG_MALLOC_OVERFLOW
//-----------------------------------------------------------------------------

#ifdef DEBUG_MALLOC
   // Allocation scurise : vrifie que l'allocation s'est bien passe,
   // affiche une erreur sinon
   void* MallocSecurise (size_t size, const char* nom_fich, const unsigned long num_ligne);
   void* ReallocSecurise (void* ptr, size_t size, const char* nom_fich, const unsigned long num_ligne);
   void* CallocSecurise (size_t nitems, size_t size, const char* nom_fich, const unsigned long num_ligne);
   char* StrdupSecurise (const char* src, const char* nom_fich, const unsigned long num_ligne);

   // Macro qui rajoutent automatiquement le nom du fichier et la ligne
#  define malloc(size) MallocSecurise(size, __FILE__, __LINE__)
#  define calloc(nitems,size) CallocSecurise(nitems, size, __FILE__, __LINE__)
#  define realloc(ptr,size) ReallocSecurise(ptr, size, __FILE__, __LINE__)
#  define strdup(src) StrdupSecurise(src, __FILE__, __LINE__)
#endif

//-----------------------------------------------------------------------------

#ifdef DEBUG_MALLOC_CONTENEUR
  // Vrifie qu'on dbogue Malloc
# ifndef DEBUG_MALLOC
#  error "Le conteneur malloc ncessite #define DEBUG_MALLOC:halloc.h"
# endif

  // Informations crites avant les donnes du pointeur
  typedef struct
  {
    unsigned long nbr_magique;       // Nombre magique
    Eina_Bool est_alloue;                 // Pointeur allou ou libr ?
    size_t taille;                   // Taille alloue (sans ces informations)
    const char* nom_fich;            // Nom du fichier source
    unsigned long num_ligne;         // Ligne du fichier
    struct PtrChaineMalloc *ptr_liste;  // Pointeur dans la liste
    unsigned long somme_info;        // Somme des informations (checksum)
  } InfoMalloc;

  // Conteneur : liste des pointeurs alloues
  // Un pointeur chan
  struct PtrChaineMalloc
  {
    struct PtrChaineMalloc *precedent;    // Elment suivant
    struct PtrChaineMalloc *suivant;      // Elment suivant
    unsigned long nbr_magique;            // Nombre magique
    InfoMalloc *ptr_info;                 // Pointeur  librer
  };

  // Ajoute un pointeur dans le conteneur malloc
  void ConteneurMalloc_Ajoute (InfoMalloc* info);

  // Affiche le contenu du conteneur malloc (libre les pointeurs si
  // libere_mem=true)
  void ConteneurMalloc_Affiche (Eina_Bool libere_mem);

  // Sort un pointeur du conteneur malloc
  void ConteneurMalloc_Remplace (InfoMalloc *Info, InfoMalloc *NvInfo);

  // Sort un pointeur du conteneur malloc
  void ConteneurMalloc_Efface (struct PtrChaineMalloc *ptr);

  // Calcule la somme de vrification (checksum) des donnes d'un pointeur
  unsigned long MallocCalculeSomme (const InfoMalloc *Info);

  // Fonction qui crit les informations d'un pointeur et modifie son adresse
  // pour qu'il pointe sur les donnes
  void *MallocPreparePtr (InfoMalloc *Info, size_t size,
                          const char *nom_fich, unsigned long num_ligne);

  // Libration scurise : vrifie que le pointeur soit valide
    void FreeSecurise (const void *ptr);

  // Macro qui rajoutent automatiquement le nom du fichier et la ligne
# define free(ptr) FreeSecurise(ptr, __FILE__, __LINE__)
#endif

//-----------------------------------------------------------------------------

// Tampon anti-overflow (criture en dehors d'une zone mmoire alloue)
#ifdef DEBUG_MALLOC_OVERFLOW
   // Vrifie que le conteneur soit activ
#  ifndef DEBUG_MALLOC_CONTENEUR
#    error "L'anti-overflow ncessite '#define DEBUG_MALLOC_CONTENEUR'"
#  endif
#endif

//-----------------------------------------------------------------------------
#endif
