/*
 * items.c
 * This file is part of FriLogos
 *
 * Copyright (C) 2012 - Alexandre Dussart
 *
 * FriLogos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * FriLogos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FriLogos. If not, see <http://www.gnu.org/licenses/>.
 */


#include <Efreet.h>
#include <Eet.h>
#include <Ecore_Evas.h>
#include <Ecore_File.h>

#include "libfrilogos.h"


void group_name_set(Group_Info *group, const char *name)
{
	if(name)
		group->name = strdup(name);
	else
		group->name = strdup(_("undefined"));
}


//
//Malloc item informations file.
//
Group_Info *group_new()
{
	Group_Info *group;

    group = malloc(sizeof(*group));
	group->id=0;
	group_name_set(group, NULL);
    group->subgroups = NULL;

    return group;
}


//
//Add a group to MainUI struct.
//If group is already there don't add it.
//
Eina_List *groups_list_get()
{
   	Item_Info *item;
   	Eina_List *items, *groups=NULL;
   	Eina_List *l;

    items = items_list_get();

	EINA_LIST_FOREACH(items, l, item)
	{
		char *s;
		Eina_Bool add = EINA_TRUE;
		Group_Info *group;

		//Parse groups.
   	    Eina_List *l2;
		EINA_LIST_FOREACH(groups, l2, group)
		{
			//If group if already here, don' add it!.
			if(strstr(item->group, group->name))
			{
				add = EINA_FALSE;
				break;
			}
		}
		if (add == EINA_TRUE)
		{
            group = group_new();
            group_name_set(group, item->group);
			group->subgroups = NULL;

				if (eina_error_get())
			  	{
					CRITICAL(_("Can't allocate list node!"));
					exit(-1);
				}

            //INF(_("Registering %s group."), group->name);
			groups = eina_list_append(groups, group);
		}

		//Check if we need to add item's subgroup to group.
		add = EINA_TRUE;

		EINA_LIST_FOREACH(group->subgroups, l2, s)
		{
			//If subgroup if already here, don' add it!.
			if(strstr(item->subgroup, s))
			{
				add = EINA_FALSE;
				break;
			}
		}

		if(add == EINA_TRUE)
		{
           //INF(_("Registering %s subgroup to %s group."),  item->subgroup, group->name);
			group->subgroups = eina_list_append(group->subgroups, strdup(item->subgroup));
				if (eina_error_get())
			  	{
					CRITICAL(_("Can't allocate list node!"));
					exit(-1);
				}
		}
	}

    items = items_list_free(items);

	groups = eina_list_sort(groups, eina_list_count(groups), EINA_COMPARE_CB(strcoll));
	//INF(_("%d groups registered."), eina_list_count(groups));

	return groups;
}




//
//Useful to convert old format to new one when needed.
//Needs to be updated to latest eet file informations format.
/*
static int _convert_old_to_new_format(const char *filename)
{
	unsigned int w, h;
    int a, c, q, l;
  	Eet_File *ef;

    //First read file.
	ef = eet_open(filename, EET_FILE_MODE_READ_WRITE);
	if(ef == NULL)
	{
		ERR(_("Can't open Eet file %s!"), filename);
		eet_close(ef);
		return -1;
	}

    INF("Adding frilogos marking/versionning field ...");
    eet_write(ef, "frilogositem", PACKAGE_VERSION, strlen(PACKAGE_VERSION)+1, 1);
    INF("Added frilogos marking/versionning field to file %s.", filename);


    INF("Updating AbstractPictureFileName and ReadPictureFileName fields...");
  	Eet_Image image;
	image.data = eet_data_image_read(ef, "Realistic Image", &w, &h, &a, &c, &q, &l);
	if(image.data)
	{
	    eet_data_image_write(ef, "images/0", image.data, w, h, a, 9, 0, 0);
	    eet_delete(ef, "Realistic Image");
        INF("Updated \"Realistic Image\" field from %s file to new format.", filename);
	}

	image.data = eet_data_image_read(ef, "Abstract Image", &w, &h, &a, &c, &q, &l);
	if(image.data)
	{
	    eet_data_image_write(ef, "images/1", image.data, w, h, a, 9, 0, 0);
	    eet_delete(ef, "Abstract Image");
        INF("Updated \"Abstract Image\" field from %s file to new format.", filename);
	}

    eet_close(ef);

    return 0;
}
*/


//
//Read all items informations files.
//
Eina_List *items_list_get()
{
	char s[PATH_MAX];
	const Eina_File_Direct_Info *f_info;
	Eina_Iterator *it;
	Eina_List *items = NULL;
	Item_Info *item = NULL;


	snprintf(s, sizeof(s), "%sitems"DIR_SEPARATOR_S, frilogos_user_data_path_get());
	it = eina_file_stat_ls(s);

    if(it)
    {
	    EINA_ITERATOR_FOREACH(it, f_info)
        {
		    if(eina_str_has_extension(f_info->path, ".eet") == EINA_TRUE)
		    {
		        //INF("Found %s new file.", ecore_file_file_get(f_info->path));
	            //_convert_old_to_new_format(f_info->path);
                item = item_read(f_info->path);

    			if(item)
    			{
    				items = eina_list_append(items, item);

    				if (eina_error_get())
    				{
    					CRITICAL(_("Can't allocate list node!"));
    					exit(-1);
    				}
    			}
    		}
    	}
	eina_iterator_free(it);
    }

	items = eina_list_sort(items, eina_list_count(items), EINA_COMPARE_CB(strcoll));
	//INF(_("%d items registered."), eina_list_count(items));

	return items;
}


/*
//
//Convert old FriLogos sqlite database to new eet files.
//
int convert_sqlitedb_to_eet(sqlite3 *db)
{

	Item_Info *itemr;
	Item_Info *itemw;
	Group_Info *cat;
	Eina_List *l;
	Eina_List *groups = NULL;

//ReadItemsTable should be defined in external tool convert.

	ReadItemsTable(db);


	EINA_LIST_FOREACH(Items, l, itemr)
	{
	    group = malloc(sizeof(*itemw));
		itemw->filename = NULL;
		item_name_set(item, item->name);
		itemw->locale = strdup("fr");

	    group = malloc(sizeof(*cat));
		categories = SelectCategoryWithID(itemr->CategoryID, db);

		cat = eina_list_data_get(groups);

		itemw->Category = strdup(cat->Name);
		itemw->SubCategory = NULL;

//ReturnItemPicturePath should be defined in external tool convert.
//

		itemw->RealistPictureFileName = strdup(ReturnItemPicturePath(itemr, 0));
		itemw->AbstractPictureFileName = strdup(ReturnItemPicturePath(itemr, 1));


		WriteItem_Info(MainUI, itemw);
		free(itemw);
		free(cat);
	}

	return EINA_TRUE;
}
*/


//
//Free item allocated struct.
//
Item_Info *item_free(Item_Info *item)
{
    if(item)
    {
      	//INF("Free \"%s\"", item->name);
	    if(item->filename)
	    free(item->filename);

	    if(item->name)
	    free(item->name);

	    if(item->locale)
	    free(item->locale);

	    if(item->group)
	    free(item->group);

	    if(item->subgroup)
        free(item->subgroup);

    	int i = 0;
        for(i = 0; i < 2; i++)
        {
       		if(item->image_filename[i])
            free(item->image_filename[i]);
        }

		if(item->sound_filename)
	    free(item->sound_filename);

        char *s;
        EINA_LIST_FREE(item->graphemics, s)
            free(s);

        EINA_LIST_FREE(item->neutrals, s)
            free(s);

        EINA_LIST_FREE(item->visuals, s)
            free(s);
            
        EINA_LIST_FREE(item->semantics, s)
            free(s);
                        
		if(item->creation_date)
	    free(item->creation_date);

		if(item->revision_date)
	    free(item->revision_date);

	    if(item->version)
        free(item->version);

	    free(item);

	    return NULL;
    }

    return NULL;
}


//
//Update item informations file.
//
int item_update(Item_Info *item)
{
	int i;
	int w, h, a;
	char s[PATH_MAX];
	void *data;
  	Eet_File *ef;
	time_t timestamp;
	struct tm *t;
	Evas *evas;
	Ecore_Evas *ee;
  	Evas_Object *im;
  	Eina_List *l;
  	char *dis;
  	
	ef = eet_open(item->filename, EET_FILE_MODE_READ_WRITE);

	if(!ef)
	{
		ERR(_("Can't open Eet file %s!"), item->filename);
		eet_close(ef);
		return -1;
	}
    eet_write(ef, "frilogositem", PACKAGE_VERSION, strlen(PACKAGE_VERSION)+1, 1);
	eet_write(ef, "name", item_name_get(item), strlen(item_name_get(item)) + 1, 1);
	eet_write(ef, "locale", item_locale_get(item), strlen(item_locale_get(item)) + 1, 1);
	eet_write(ef, "group", item_group_get(item), strlen(item_group_get(item)) + 1, 1);
	eet_write(ef, "subgroup", item_subgroup_get(item), strlen(item_subgroup_get(item)) + 1, 1);
	eet_write(ef, "sounds/0", item_sound_filename_get(item), strlen(item_sound_filename_get(item)) + 1, 1);


	ee = ecore_evas_buffer_new(1, 1);
	if (!ee)
	  ERR(_("Can't create Ecore_Evas buffer!"));

	evas = ecore_evas_get(ee);

	//Update item pictures.
	for(i = 0; i < 2; i++)
	{
		if(item->image_filename[i])
		{
			im = evas_object_image_add(evas);
			evas_object_image_file_set(im, item->image_filename[i], NULL);
			data = evas_object_image_data_get(im, EINA_FALSE);
			evas_object_image_scale(im, 400, 400);
			evas_object_image_size_get(im, &w, &h);
			a = evas_object_image_alpha_get(im);

			snprintf(s, sizeof(s), "images/%d", i);
			eet_data_image_write(ef, s, data, w, h, a, 9, 0, 0);
			evas_object_del(im);
		}
	}

	ecore_evas_free(ee);
	evas_free(evas);

    //Delete current item's distractors.
    int num;
    char **list = eet_list(ef, "distractors/*", &num);
    if(list)
    {
        for (i = 0; i < num; i++)
            eet_delete (ef, list[i]);
        free(list);
    }    
        
    //Add item's distractors.
    i = 0;
	EINA_LIST_FOREACH(item->graphemics, l, dis)
	{
		snprintf(s, sizeof(s), "distractors/graphemics/%d", i);	
    	eet_write(ef, s, strdup(dis), strlen(dis) + 1, 1);
    	i++;
	}

    i = 0;		
	EINA_LIST_FOREACH(item->neutrals, l, dis)
    {
		snprintf(s, sizeof(s), "distractors/neutrals/%d", i);	
    	eet_write(ef, s, strdup(dis), strlen(dis) + 1, 1);
    	i++;
	}
	
    i = 0;	   
	EINA_LIST_FOREACH(item->visuals, l, dis)
    {
		snprintf(s, sizeof(s), "distractors/visuals/%d", i);	
    	eet_write(ef, s, strdup(dis), strlen(dis) + 1, 1);
    	i++;
	}

    i = 0;		      
	EINA_LIST_FOREACH(item->semantics, l, dis)
    {
		snprintf(s, sizeof(s), "distractors/semantics/%d", i);	
    	eet_write(ef, s, strdup(dis), strlen(dis) + 1, 1);
    	i++;
	}

	//Update item revision date informations.
	timestamp = time(NULL);
	t = localtime(&timestamp);

	snprintf(s, sizeof(s), "%02d-%02d-%d",
				(int)t->tm_mday,
  				(int)t->tm_mon,
  				1900+(int)t->tm_year);
	eet_write(ef, "revisiondate", s, strlen(s) + 1, 1);
	item->revision_date = strdup(s);

    eet_sync(ef);
	eet_close(ef);

	return 0;
}


//
//Write item informations file.
//Return item filename if success.
const char  *item_write(Item_Info *item)
{
	int i = 0;
	int w, h, a;
	char s[PATH_MAX];
	char *dis;
	void *data;
  	Eet_File *ef;
	time_t timestamp;
	struct tm *t;
	Evas *evas;
	Ecore_Evas *ee;
  	Evas_Object *im;
  	Eina_List *l;


	snprintf(s, sizeof(s), "%s"DIR_SEPARATOR_S"items"DIR_SEPARATOR_S"%s.eet", frilogos_user_data_path_get(), ecore_file_file_get(item->name));

	while(ecore_file_exists(s) == EINA_TRUE)
	{
		//strftime(buf, sizeof(buf),"%H%M%S", t);
		snprintf(s, sizeof(s), "%s"DIR_SEPARATOR_S"items"DIR_SEPARATOR_S"%s-%03d.eet", frilogos_user_data_path_get(), ecore_file_file_get(item->name), i);
		i++;
	}

	item->filename = strdup(s);
	ef = eet_open(item->filename, EET_FILE_MODE_WRITE);

	if(ef == NULL)
	{
		ERR(_("Can't open Eet file %s!"), item->filename);
		eet_close(ef);
		return NULL;
	}
	
    eet_write(ef, "frilogositem", PACKAGE_VERSION, strlen(PACKAGE_VERSION)+1, 1);
	eet_write(ef, "name", item_name_get(item), strlen(item_name_get(item)) + 1, 1);
	eet_write(ef, "locale", item_locale_get(item), strlen(item_locale_get(item)) + 1, 1);
	eet_write(ef, "group", item_group_get(item), strlen(item_group_get(item)) + 1, 1);
	eet_write(ef, "subgroup", item_subgroup_get(item), strlen(item_subgroup_get(item)) + 1, 1);
	eet_write(ef, "sounds/0", item_sound_filename_get(item), strlen(item_sound_filename_get(item)) + 1, 1);

	ee = ecore_evas_buffer_new(1, 1);
	if (!ee)
	  ERR(_("Can't create Ecore_Evas buffer!"));

	evas = ecore_evas_get(ee);

	//Add item pictures.
	for(i = 0; i < 2; i++)
	{
        im = evas_object_image_add(evas);
        if(item->image_filename[i])
        {
            evas_object_image_file_set(im, item->image_filename[i], NULL);
        }
        else
        {
			snprintf(s, sizeof(s), "%s/icons/nopicture.png", frilogos_system_data_path_get());
			evas_object_image_file_set(im, s, NULL);
		}

		data = evas_object_image_data_get(im, EINA_FALSE);
		//evas_object_image_scale(im, 400, 400);
		evas_object_image_size_get(im, &w, &h);
		a = evas_object_image_alpha_get(im);

		snprintf(s, sizeof(s), "images/%d", i);
		eet_data_image_write(ef, s, data, w, h, a, 9, 0, 0);
		evas_object_del(im);
	}

	ecore_evas_free(ee);
	evas_free(evas);

    //Add item distractors.
    i = 0;
	EINA_LIST_FOREACH(item->graphemics, l, dis)
	{
		snprintf(s, sizeof(s), "distractors/graphemics/%d", i);	
    	eet_write(ef, s, strdup(dis), strlen(dis) + 1, 1);
    	i++;
	}

    i = 0;		
	EINA_LIST_FOREACH(item->neutrals, l, dis)
    {
		snprintf(s, sizeof(s), "distractors/neutrals/%d", i);	
    	eet_write(ef, s, strdup(dis), strlen(dis) + 1, 1);
    	i++;
	}
	
    i = 0;	   
	EINA_LIST_FOREACH(item->visuals, l, dis)
    {
		snprintf(s, sizeof(s), "distractors/visuals/%d", i);	
    	eet_write(ef, s, strdup(dis), strlen(dis) + 1, 1);
    	i++;
	}

    i = 0;		      
	EINA_LIST_FOREACH(item->semantics, l, dis)
    {
		snprintf(s, sizeof(s), "distractors/semantics/%d", i);	
    	eet_write(ef, s, strdup(dis), strlen(dis) + 1, 1);
    	i++;
	}
    
	//Add item creation informations.
	timestamp = time(NULL);
	t = localtime(&timestamp);

	snprintf(s, sizeof(s), "%02d-%02d-%d",
				(int)t->tm_mday,
  				(int)t->tm_mon,
  				1900+(int)t->tm_year);

	item->creation_date = strdup(s);
	eet_write(ef, "creationdate", s, strlen(s) + 1, 1);

	item->revision_date = NULL;

	eet_sync(ef);
	eet_close(ef);

	return strdup(item->filename);
}


//
//Delete item informations file.
//
int item_remove(Item_Info *item)
{
    //INF(_("Removing:%s"), item->filename);
	if(ecore_file_remove(item->filename) == EINA_FALSE)
	{
	    ERR(_("Can't remove %s!"), item->filename);
	    return -1;
	}

	return 0;
}


//
//Malloc item informations file.
//
Item_Info *item_new()
{
	Item_Info *item;

    item = malloc(sizeof(*item));
	item->filename=NULL;
	item_name_set(item, NULL);
    item_locale_set(item, "fr");
    item_group_set(item, NULL);
    item_subgroup_set(item, NULL);
	item_image_filename_set(item, NULL, 0);
	item_image_filename_set(item, NULL, 1);
	item_sound_filename_set(item, NULL);
    item->neutrals = NULL;
    item->visuals = NULL;
    item->semantics = NULL;
    item->graphemics = NULL;    
	item->creation_date=NULL;
	item->revision_date=NULL;
	item->version=NULL;

    return item;
}


//
//Import a new item, simply copy it to FriLogos data items dir.
//
int item_import(Item_Info *item)
{
    int i = 0;
    char dst[PATH_MAX];

	snprintf(dst, sizeof(dst), "%s"DIR_SEPARATOR_S"items"DIR_SEPARATOR_S"%s.eet", frilogos_user_data_path_get(), item->name);
	while(ecore_file_exists(dst) == EINA_TRUE)
	{
	    snprintf(dst, sizeof(dst), "%s"DIR_SEPARATOR_S"items"DIR_SEPARATOR_S"%s-%03d.eet", frilogos_user_data_path_get(), item->name, i);
		i++;
	}

    //INF(_("Import from %s to %s."), item->filename, dst);
    if(ecore_file_cp(item->filename, dst) == EINA_FALSE)
    {
        ERR(_("Can't import %s!"), item->name);
        return -1;
    }

    return 0;
}


//
//Export an item, simply copy it to speficied dst dir.
//
int item_export(Item_Info *item, const char *dst)
{
    int i = 0;
    char s[PATH_MAX];


	snprintf(s, sizeof(s), "%s/%s.eet", dst, item->name);
	while(ecore_file_exists(s) == EINA_TRUE)
	{
	    snprintf(s, sizeof(s), "%s/%s-%03d.eet", dst, item->name,i++);
	}

    INF(_("Export %s to %s."), item->filename, s);
    if(ecore_file_cp(item->filename, s) == EINA_FALSE)
    {
        ERR(_("Can't export %s!"), item->name);
        return -1;
    }

    return 0;
}


//
//Read item informations file.
//
Item_Info *item_read(const char *f)
{
	int i;
	int size, num;
	char *ret, **list;
  	Eet_File *ef;
	Item_Info *item;

	ef = eet_open(f, EET_FILE_MODE_READ);
	if(ef == NULL)
	{
		ERR(_("Can't open Eet file %s!"), f);
		eet_close(ef);
		return NULL;
	}
	//INF(_("Reading %s."), f);
	
	//Check if it's a FriLogos item by reading frilogositem special field.
    ret = eet_read(ef, "frilogositem", &size);
        if(!ret)
        {
        	ERR(_("Can't read FriLogos Eet file %s!"), f);
        	return NULL;
        }
	item = malloc(sizeof(*item));
    item->version = strdup(ret);
        
        
    free(ret);
	
	
	//Read text informations about item.
	item->filename = strdup(f);
    ret = eet_read(ef, "name", &size);
    item_name_set(item, ret);
    free(ret);

    ret = eet_read(ef, "locale", &size);
    item_locale_set(item, ret);
    free(ret);

	ret = eet_read(ef, "sounds/0", &size);
    if(ret)
    	item->sound_filename = strdup(ret);
	else
		item->sound_filename = NULL;
    free(ret);

	for(i = 0; i < 2; i++)
		item->image_filename[i] = NULL;

    ret = eet_read(ef, "group", &size);
    item_group_set(item, ret);
    free(ret);

    ret = eet_read(ef, "subgroup", &size);
    item_subgroup_set(item, ret);
    free(ret);

	//Read item distractors.
    item->neutrals = NULL;
    item->visuals = NULL;
    item->semantics = NULL;
    item->graphemics = NULL;  

    list = eet_list(ef, "distractors/*", &num);
    if(list)
    {
    	Distractor_Type dis = UNDEFINED;    
    	
        for (i = 0; i < num; i++)
        {
            ret = eet_read(ef, list[i], &size);        
            if(strstr(list[i],"graphemics/"))
                dis = GRAPHEMIC;
            if(strstr(list[i],"neutrals/"))
                dis = NEUTRAL;
            if(strstr(list[i],"semantics/"))
                dis = SEMANTIC;
            if(strstr(list[i],"visuals/"))
                dis = VISUAL;

            item_distractor_add(item, strdup(ret), dis);  
            free(ret);  
        }
        free(list);
    }

    ret = eet_read(ef, "creationdate", &size);
	item->creation_date = strdup(ret);
    free(ret);

	ret = eet_read(ef, "revisiondate", &size);
	if(size > 0)
		item->revision_date = strdup(ret);
	else
		item->revision_date = NULL;
    free(ret);

	//INF(_("%d fields read in %s file."), eet_num_entries(ef), item->filename);

	eet_close(ef);

	item->neutrals = eina_list_sort(item->neutrals, eina_list_count(item->neutrals), EINA_COMPARE_CB(strcoll));
	item->semantics = eina_list_sort(item->semantics, eina_list_count(item->semantics), EINA_COMPARE_CB(strcoll));
	item->visuals = eina_list_sort(item->visuals, eina_list_count(item->visuals), EINA_COMPARE_CB(strcoll));
	item->graphemics = eina_list_sort(item->graphemics, eina_list_count(item->graphemics), EINA_COMPARE_CB(strcoll));		

	return item;
}




Eina_List *items_list_randomize_from_group(Eina_List *items, const char *group)
{
	Eina_List *l;
	Eina_List *ret = NULL;
	Item_Info *item;

	INF(_("Randomize items from group %s."), group);
	EINA_LIST_FOREACH(items, l, item)
	{
		if(strcmp(item->group, group) == 0)
			ret = eina_list_append(ret, item);
	}

	return ret;
}


static int sort_cb()
{
    return Prandom(2) - 1;
}


Eina_List *items_list_randomize(const Eina_List *items)
{
	Eina_List *ret = NULL;

	INF(_("Randomize items from all groups."));

	ret = eina_list_clone(items);
	ret = eina_list_sort(ret, eina_list_count(ret), EINA_COMPARE_CB(sort_cb));

	return ret;
}



Eina_List *items_list_randomize_with_group_and_subgroup(Eina_List *items, const char *group, const char *subgroup)
{
	Eina_List *l;
	Eina_List *ret = NULL;
	Item_Info *item;

	INF(_("Randomize items from group %s and subgroup %s."), group, subgroup);
	EINA_LIST_FOREACH(items, l, item)
	{
		if((strcmp(item->group, group) == 0) && (strcmp(item->subgroup, subgroup) == 0))
		{
			ret = eina_list_append(ret, item);
		}
	}

	ret = eina_list_sort(ret, eina_list_count(ret), EINA_COMPARE_CB(sort_cb));

	return ret;
}




Eina_List *items_list_randomize_except(Eina_List *items, const char *name)
{
	Eina_List *l;
	Eina_List *ret = NULL;
	Item_Info *item;

	INF(_("Randomize items except %s from any groups."), name);
	EINA_LIST_FOREACH(items, l, item)
	{
		if(strcmp(item->name, name) != 0)
			ret = eina_list_append(ret, item);
	}

	ret = eina_list_sort(ret, eina_list_count(ret), EINA_COMPARE_CB(sort_cb));

	return ret;
}



Eina_List *items_list_randomize_except_from_group(Eina_List *items, const char *group, const char *name)
{
	Eina_List *l;
	Eina_List *ret = NULL;
	Item_Info *item;

	INF(_("Randomize items except %s from group %s."), name, group);
	EINA_LIST_FOREACH(items, l, item)
	{
		if((strcmp(item->name, name) != 0) && (strcmp(item->group, group) == 0))
			ret = eina_list_append(ret, item);
	}

	ret = eina_list_sort(ret, eina_list_count(ret), EINA_COMPARE_CB(sort_cb));

	return ret;
}


//
//Accessors functions.
//
const char *item_name_get(Item_Info *item)
{
	if(item->name)
        return item->name;
	else
		return _("undefined");
}

const char *item_locale_get(Item_Info *item)
{
	if(item->locale)
        return item->locale;
	else
		return _("undefined");
}


const char *item_group_get(Item_Info *item)
{
	if(item->group)
        return item->group;
	else
		return _("undefined");
}


const char *item_subgroup_get(Item_Info *item)
{
	if(item->subgroup)
        return item->subgroup;
	else
		return _("undefined");
}


const char  *item_sound_filename_get(Item_Info *item)
{
	if(!item->sound_filename)
       	return _("undefined");
	else
        return item->sound_filename;
}


void item_name_set(Item_Info *item, const char *name)
{
	if(name)
		item->name = strdup(name);
	else
		item->name = strdup(_("undefined"));
}


void item_locale_set(Item_Info *item, const char *locale)
{
	if(locale)
		item->locale = strdup(locale);
	else
		item->locale = strdup(_("undefined"));
}


void item_image_filename_set(Item_Info *item, const char *filename, const int level)
{
    if(level>1)
    {
        ERR(_("Can't define item image filename!"));
        return;
    }

    if((filename) && (ecore_file_exists(filename)==EINA_TRUE))
    {
    	item->image_filename[level]=strdup(filename);
    }
    else
    {
		item->image_filename[level] = NULL;
    }
}


//
//TODO:implement item sound handling.
//
void item_sound_filename_set(Item_Info *item, const char *filename)
{

    item->sound_filename = NULL;

    if((filename) && (ecore_file_exists(filename)==EINA_TRUE))
    {
    	item->sound_filename=strdup(filename);
    }
    else
    {
    	item->sound_filename = NULL;
    }
}

/*
Eina_List *item_distractors_randomize(item Distractor_Type type)
{


}
*/


void item_distractor_clear(Item_Info *item, Distractor_Type type)
{
    Eina_List *l;
    char *s;
    switch(type)
    {
        case NEUTRAL:
            	EINA_LIST_FOREACH(item->neutrals, l, s)
                        item->neutrals  = eina_list_remove(item->neutrals, s); 	
            break;
            
        case GRAPHEMIC:
            	EINA_LIST_FOREACH(item->graphemics, l, s)
                        item->graphemics  = eina_list_remove(item->graphemics, s);
            break;        
            
        case VISUAL:
            	EINA_LIST_FOREACH(item->visuals, l, s)
                        item->visuals  = eina_list_remove(item->visuals, s);
            break;  
            
        case SEMANTIC:
            	EINA_LIST_FOREACH(item->semantics, l, s)
                        item->semantics  = eina_list_remove(item->semantics, s);
            break;     
                    
        default:
            ERR(_("Can't clear item's distractors of an unknown type!"));
            break;
    }
}



void item_distractor_add(Item_Info *item, const char *name, Distractor_Type type)
{
    Eina_List *l;
    char *s;
    
    //Don't try to add if already registered.
	EINA_LIST_FOREACH(item->neutrals, l, s)
	{
	    if(strcmp(s,name) == 0)
	        return;
	}

	EINA_LIST_FOREACH(item->graphemics, l, s)
	{
	    if(strcmp(s, name) == 0)
	        return;
	}
	
	EINA_LIST_FOREACH(item->visuals, l, s)
	{
	    if(strcmp(s, name) == 0)
	        return;
	}
	
	EINA_LIST_FOREACH(item->semantics, l, s)
	{
	    if(strcmp(s, name) == 0)
	        return;
	}		

    switch(type)
    {
        case NEUTRAL:
                item->neutrals = eina_list_append(item->neutrals, strdup(name));
            break;
            
        case GRAPHEMIC:    
                item->graphemics = eina_list_append(item->graphemics, strdup(name));        
            break;        
            
        case VISUAL:
                item->visuals = eina_list_append(item->visuals, strdup(name));        
            break;  
            
        case SEMANTIC:
                item->semantics = eina_list_append(item->semantics, strdup(name));        
            break;     
                    
        default:
            ERR(_("Can't add an item's distractor with an unknown type!"));
            break;
    }
}



//
//
//
void item_group_set(Item_Info *item, const char *group)
{
	if(group)
		item->group = strdup(group);
	else
		item->group = strdup(_("undefined"));
}


void item_subgroup_set(Item_Info *item, const char *subgroup)
{
	if(subgroup)
		item->subgroup = strdup(subgroup);
	else
		item->subgroup = strdup(_("undefined"));
}


//
//Debug print item.
//
void item_print(Item_Info *item)
{
	printf(_("\nFilename:%s\n"
		"\t|->Name:%s\n"\
		"\t|->Locale:%s\n"
		"\t|->Group:%s\n"
		"\t|->SubGroup:%s\n"
		"\t|->Creation Date:%s\n"
		"\t|->Revision Date:%s\n"),
		ecore_file_file_get(item->filename),
		item->name,
		item->locale,
		item->group,
		item->subgroup,
		item->creation_date,
		item->revision_date);
		
    item_distractors_print(item);
}



void item_distractors_print(const Item_Info *item)
{
    Eina_List *l = NULL;
    char *s;

	EINA_LIST_FOREACH(item->graphemics, l, s)
	   printf(_("\t|Graphemics->%s\n"), s);

	EINA_LIST_FOREACH(item->neutrals, l, s)
	   printf(_("\t|Neutrals->%s\n"), s);
	   
	EINA_LIST_FOREACH(item->visuals, l, s)
	   printf(_("\t|Visuals->%s\n"), s);
	      
	EINA_LIST_FOREACH(item->semantics, l, s)
	   printf(_("\t|Semantics->%s\n"), s);
	
}


//
//Print items list.
//
void items_list_print(const Eina_List *items)
{

    Eina_List *l = NULL;
    Item_Info *item;


	EINA_LIST_REVERSE_FOREACH(items, l, item)
	{
      printf("\t->%s\n", item->name);
      //item_print(item);
    }

	//INF(_("%d items registered."), eina_list_count(items));
}


//
//Free items list.
//
Eina_List * items_list_free(Eina_List *items)
{
	if(items)
	{
    	Item_Info *item;

        //Point to first node of list.
        for(items = eina_list_last(items); items; items = eina_list_prev(items));

        EINA_LIST_FREE(items, item)
            item = item_free(item);

	   //INF(_("%d items registered."), eina_list_count(items));
	    return NULL;
    }

    return NULL;
}


//
//Free group definition.
//
Group_Info *group_free(Group_Info *group)
{
    if(group)
    {
        free(group->name);
        group->name = NULL;

        for(group->subgroups = eina_list_last(group->subgroups); group->subgroups; group->subgroups = eina_list_prev(group->subgroups));

        char *s;
        EINA_LIST_FREE(group->subgroups, s)
        {
            free(s);
            s = NULL;
        }
        return NULL;
    }

    return NULL;
}


//
//Free groups list.
//
Eina_List *groups_list_free(Eina_List *groups)
{
	if(!groups)
	{
		Group_Info *group;

        //Point to first node of list.
        for(groups = eina_list_last(groups); groups; groups = eina_list_prev(groups));

        EINA_LIST_FREE(groups, group)
            group = group_free(group);
	    
	    //INF(_("%d groups registered."), eina_list_count(groups));

        return NULL;
    }

    return NULL;
}


//
//Debug and test items funcs.
//
void items_list_debug_test()
{
	Eina_List *items;
	Eina_List *l;

    //Create a new 'arteface' item.
    Item_Info *item;
    
    item = item_new();
    item_name_set(item, "Artefact");
    item_group_set(item, "objets");
    item_distractor_add(item, "Visu0", VISUAL);
    item_distractor_add(item, "Visu1", VISUAL);
    item_distractor_add(item, "Semant0", SEMANTIC);    
    item_distractor_add(item, "Semant0", SEMANTIC);    
    item_distractor_add(item, "Semant1", SEMANTIC);    
    item_distractor_add(item, "Semant2", SEMANTIC);    
    item_distractor_add(item, "Neutr0", NEUTRAL);    
    item_distractor_add(item, "Neutr1", NEUTRAL);    
    item_distractor_add(item, "Graph0", GRAPHEMIC);    
    item_distractor_add(item, "Graph1", GRAPHEMIC);    
    item_distractor_add(item, "Graph2", GRAPHEMIC);    
    item_write(item);
    item = item_free(item);

	items = items_list_get();

	l = items_list_randomize(items);
	items_list_print(l);
	l = items_list_randomize_from_group(items, "animaux");
	items_list_print(l);
	l = items_list_randomize_from_group(items, "objets");
	items_list_print(l);
	l = items_list_randomize_from_group(items, "objets");
	items_list_print(l);
	l = items_list_randomize_from_group(items, "aliments");
	items_list_print(l);
	l = items_list_randomize_except(items, "un cheval");
	items_list_print(l);
    l= items_list_randomize_except_from_group(items, "animaux", "un cheval");
	items_list_print(l);

	Group_Info *gr;
	Eina_List *groups;
	groups = groups_list_get();

	EINA_LIST_FOREACH(groups, l, gr)
	{
		INF(_("%s group has %d subgroups."), gr->name, eina_list_count(gr->subgroups));
	}

    groups= groups_list_free(groups);
	items = items_list_free(items);
	l = items_list_free(l);
}
