/*
 * path.c
 * This file is part of FriLogos
 *
 * Copyright (C) 2012 - Alexandre Dussart
 *
 * FriLogos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * FriLogos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FriLogos. If not, see <http://www.gnu.org/licenses/>.
 */

#include <Ecore_File.h>
#include <Efreet.h>

#include "libfrilogos.h"


#ifdef HAVE_EVIL
static const char *_find_program_in_path (const char *program)
{
  int n;
  wchar_t wfilename[MAXPATHLEN], wsysdir[MAXPATHLEN], wwindir[MAXPATHLEN];
  char s[MAXPATHLEN];

  n = GetModuleFileNameW (NULL, wfilename, MAXPATHLEN);

  wcstombs(s, wfilename, wcslen(wfilename)+1);
  return strdup(s);
 }
#endif



const char *frilogos_locale_path_get(void)
{
#ifdef HAVE_EVIL
	char s[PATH_MAX];
	snprintf(s, sizeof(s), "%s"DIR_SEPARATOR_S"locale"DIR_SEPARATOR_S, frilogos_system_data_path_get());
	return strdup(s);
#else
	return PACKAGE_LOCALE_DIR;
#endif
}


// Return FriLogos user data directory. useful to get user data path.
// @return char pointer containing installation directory.
const char *frilogos_user_data_path_get (void)
{
#ifdef HAVE_EVIL
	char s[MAX_PATH];
	SHGetFolderPath(NULL, CSIDL_LOCAL_APPDATA,  NULL,  0, s);
	strcat(s, DIR_SEPARATOR_S"FriLogos.Org"DIR_SEPARATOR_S"FriLogos"DIR_SEPARATOR_S);
	return strdup(s);
#else
	char s[PATH_MAX];
	snprintf(s, sizeof(s), "%s"DIR_SEPARATOR_S"data"DIR_SEPARATOR_S"FriLogos.Org"DIR_SEPARATOR_S"FriLogos"DIR_SEPARATOR_S"", efreet_data_home_get());
	return strdup(s);
#endif
}


// Return FriLogos installation directory. useful to get global installation data path.
// @return char pointer containing installation directory.
const char *frilogos_system_data_path_get (void)
{
#ifdef HAVE_EVIL
	char s[MAX_PATH];
	snprintf(s, sizeof(s), "%s"DIR_SEPARATOR_S"share"DIR_SEPARATOR_S, ecore_file_dir_get(_find_program_in_path("frilogos.exe")));
	return strdup(s);
#else
	return PACKAGE_DATA_DIR;
#endif
}



const char *frilogos_edje_theme_file_get (void)
{
	char s[PATH_MAX];
    snprintf(s, sizeof(s), "./frilogos.edj");
    if(ecore_file_exists(s) == EINA_TRUE)
    {
            return strdup(s);
    }
    else
    {
        snprintf(s, sizeof(s), "%s/themes/default/frilogos.edj", PACKAGE_SRC_DIR);
        if(ecore_file_exists(s) == EINA_TRUE)
        {
            return strdup(s);
        }
        else
        {
            snprintf(s, sizeof(s), "%s/themes/default/frilogos.edj", frilogos_system_data_path_get());
            if(ecore_file_exists(s) == EINA_TRUE)
            {
                return strdup(s);
            }
            else
            {
                ERR(_("Can't found frilogos theme!"));
                return NULL;
            }
        }
    }
}
