/*
 * Copyright (C) Alexandre Dussart 2009-2011 <alexandre.dussart@laposte.net>
 *
 * FriLogos is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FriLogos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <time.h>
#include <Evas.h>

#include "libfrilogos.h"


static Eina_Bool FoundIn(int val, int *ptr, int n_elements);



/*
 * Initialize random number generator.
 */
void SeedRandomizer(void)
{
   time_t timestamp;
   struct tm *t;
   int setseed = 0;

   DBG("Initialize random generator...");

   timestamp = time(NULL);
   t = localtime(&timestamp);

   setseed =
      ((int)t->tm_year + (int)t->tm_mon + (int)t->tm_wday + (int)t->tm_mday +
       (int)t->tm_hour + (int)t->tm_min + (int)t->tm_sec);

   srand(setseed);
}

/*
 * Return a random number between 1 & max.
 */
int Prandom(int max)
{
 //  return ((rand() % (int)(((max) + 1) - (min))) + (min));
//	return (int)(rand() / (double)RAND_MAX * (max - 1));;
// return rand() * (max / RAND_MAX);

    int partSize   = 1 + (max == RAND_MAX ? 0 : (RAND_MAX - max) / (max + 1));
    int maxUsefull = partSize * max + (partSize - 1);
    int draw;

    do {
        draw = rand();
    } while (draw > maxUsefull);

    return draw / partSize;

}

/*
 * Return a randomized different value list.
 */
extern void
RandomizeList(int *ptr, int n_elements)
{
   int i;

	for (i = 0; i < n_elements; i++)
	{
		ptr[i] = (int)Prandom(n_elements);

		while (FoundIn(ptr[i], ptr, i) == EINA_TRUE)
		{
			ptr[i] = (int)Prandom(n_elements);
		}
		INF("%d", ptr[i]);
	}
}

static Eina_Bool
FoundIn(int val, int *ptr, int n_elements)
{
   int i;

	for (i = 0; i < n_elements; i++)
	{
		if (ptr[i] == val)
		{
			return EINA_TRUE;
		}
	}

   return EINA_FALSE;
}
