/*
 * Copyright (C) Alexandre Dussart 2009-2011 <alexandre.dussart@laposte.net>
 *
 * FriLogos is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FriLogos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <Efreet.h>
#include <ctype.h>


#include "libfrilogos.h"
#include "gettext.h"


//
//Strip white spaces of a string.
//
char *trimwhitespace(char *str)
{
  char *end;

  // Trim leading space
  while(isspace(*str)) str++;

  if(*str == 0)  // All spaces?
    return str;

  // Trim trailing space
  end = str + strlen(str) - 1;
  while(end > str && isspace(*end)) end--;

  // Write new null terminator
  *(end+1) = 0;

  return str;
}


//Keep aspect ratio.
void evas_object_image_scale(Evas_Object *obj, int width, int height)
{
	int w3, h3;
	int w2, h2;

	evas_object_image_size_get(obj, &w2, &h2);

	w3 = width;
	h3 = ((double)w3 * h2)/ (double)w2;


        if(h3 > height)
        {
                h3 = height;
                w3 = ((double)h3 * w2) / (double)h2;
        }

  	evas_object_image_fill_set(obj, 0, 0, w3, h3);
	evas_object_resize(obj, w3, h3);
}

//Return home user dir in a portable way.
const char *efreet_user_home_get(void)
{
#ifdef G_OS_WIN32
	char *home;
	home = g_strdup (g_getenv ("HOME"));

	if(!home)
	{
	  free (home);
	  home = NULL;
	}

	if (home)
	{
      		char *p;
		while ((p = strchr (home, '/')) != NULL)
		*p = '\\';
    	}

	if (!home)
	{
		if (g_getenv ("USERPROFILE") != NULL)
		home = g_strdup (g_getenv ("USERPROFILE"));
	}

	if (!home)
		home = get_special_folder (CSIDL_PROFILE);

    return home;
#else
    return(getenv ("HOME"));
#endif
}


//Close a window by deleting evas object passing arg.
void window_clicked_close_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
	evas_object_del(data);
}
