/*
 * module_settings.c
 * This file is part of Frilogos
 *
 * Copyright (C) 2012 - Alexandre Dussart
 *
 * Frilogos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Frilogos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Frilogos. If not, see <http://www.gnu.org/licenses/>.
 */


#include "module.h"


static Module_Settings *settings = NULL;

static void _nbserial_sl_changed_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__);
static void _wantnavbar_change_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__);
static void _wantgraphemics_change_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__);
static void _wantsemantics_change_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__);
static void _wantvisuals_change_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__);
static void _wanttimers_change_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__);
static void _wantitemname_change_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__);
static void _showtimer_sl_changed_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__);
static void _update_module_settings();
//static Module_Settings *_module_settings_free(Module_Settings *settings);


//
//Called when user modify number of serial wanted.
//
static void  _nbserial_sl_changed_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__)
{
    Module_Settings *settings = data;
    settings->nbserial_wanted = elm_slider_value_get(obj);

    elm_slider_value_set(obj, settings->nbserial_wanted);
  
    if(settings->patient_selected)  
        file_setting_write(settings->patient_selected->filename, "DIAPORAMA/nbserial_wanted", settings->nbserial_wanted);
}


//
//Called when user enable navigation bar.
//
static void _wantnavbar_change_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__)
{
	Module_Settings *settings = data;

	settings->navbar_wanted = elm_check_state_get(obj);
	elm_check_state_set(obj, settings->navbar_wanted);
	
    if(settings->patient_selected)
        file_setting_write(settings->patient_selected->filename, "DIAPORAMA/navbar_wanted", settings->navbar_wanted);
}


//
//Called when user enable distractors.
//
static void _wantgraphemics_change_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__)
{
	Module_Settings *settings = data;

	settings->graphemics_wanted = elm_check_state_get(obj);
	elm_check_state_set(obj, settings->graphemics_wanted);
	
    if(settings->patient_selected)
        file_setting_write(settings->patient_selected->filename, "DIAPORAMA/graphemics_wanted", settings->graphemics_wanted);
}


//
//Called when user enable distractors.
//
static void _wantsemantics_change_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__)
{
	Module_Settings *settings = data;

	settings->semantics_wanted = elm_check_state_get(obj);
	elm_check_state_set(obj, settings->semantics_wanted);
	
    if(settings->patient_selected)
        file_setting_write(settings->patient_selected->filename, "DIAPORAMA/semantics_wanted", settings->semantics_wanted);
}


//
//Called when user enable distractors.
//
static void _wantvisuals_change_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__)
{
	Module_Settings *settings = data;

	settings->visuals_wanted = elm_check_state_get(obj);
	elm_check_state_set(obj, settings->visuals_wanted);
	
    	if(settings->patient_selected)
            file_setting_write(settings->patient_selected->filename, "DIAPORAMA/visuals_wanted", settings->visuals_wanted);
}

//
//Called when user enable timers settings.
//
static void _wanttimers_change_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__)
{
	Module_Settings *settings = (Module_Settings *)data;

	settings->timers_wanted = elm_check_state_get(obj);
	elm_check_state_set(obj, settings->timers_wanted);
	
    if(settings->patient_selected)
        file_setting_write(settings->patient_selected->filename, "DIAPORAMA/timers_wanted", settings->timers_wanted);

	if(settings->timers_wanted == EINA_FALSE)
	{
		elm_object_disabled_set(settings->exposition_sl, EINA_TRUE);
		elm_object_disabled_set(settings->hiding_sl, EINA_TRUE);
	}
	else
	{
		elm_object_disabled_set(settings->exposition_sl, EINA_FALSE);
		elm_object_disabled_set(settings->hiding_sl, EINA_FALSE);
	}
}


//
//
//
static void _showtimer_sl_changed_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__)
{
    Module_Settings *settings = data;
    settings->exposition_timer_wanted = elm_slider_value_get(obj);
    
    if(settings->patient_selected)     
        file_setting_write(settings->patient_selected->filename, "DIAPORAMA/exposition_timer_wanted", settings->exposition_timer_wanted);
}

//
//
//
static void _imglvl_sc_changed_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info)
{
    Module_Settings *settings = data;
    settings->imglvl_wanted = elm_segment_control_item_index_get(event_info);
    
    if(settings->patient_selected)   
        file_setting_write(settings->patient_selected->filename, "DIAPORAMA/imglvl_wanted", settings->imglvl_wanted);
}

   
   
//
//
//
static void _hidetimer_sl_changed_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__)
{
    Module_Settings *settings = data;
    settings->hiding_timer_wanted = elm_slider_value_get(obj);

    if(settings->patient_selected)
        file_setting_write(settings->patient_selected->filename, "DIAPORAMA/hiding_timer_wanted", settings->hiding_timer_wanted);
}


//
//Called when user enable item label.
//
static void _wantitemname_change_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__)
{
	Module_Settings *settings = data;

	settings->itemlabel_wanted = elm_check_state_get(obj);
	elm_check_state_set(obj, settings->itemlabel_wanted);

    if(settings->patient_selected)
        file_setting_write(settings->patient_selected->filename, "DIAPORAMA/itemlabel_wanted", settings->itemlabel_wanted);

	if(settings->itemlabel_wanted == EINA_TRUE)
	{
		elm_check_state_set(settings->navigationbar_checkb, EINA_TRUE);
		elm_object_disabled_set(settings->navigationbar_checkb, EINA_TRUE);
		elm_check_state_set(settings->semantics_checkb, EINA_FALSE);
		elm_check_state_set(settings->graphemics_checkb, EINA_FALSE);
		elm_check_state_set(settings->visuals_checkb, EINA_FALSE);
		elm_object_disabled_set(settings->semantics_checkb, EINA_TRUE);
		elm_object_disabled_set(settings->graphemics_checkb, EINA_TRUE);
		elm_object_disabled_set(settings->visuals_checkb, EINA_TRUE);
		settings->distractors_wanted = EINA_FALSE;
	}
	else
	{
		elm_object_disabled_set(settings->semantics_checkb, EINA_FALSE);
		elm_object_disabled_set(settings->graphemics_checkb, EINA_FALSE);
		elm_object_disabled_set(settings->visuals_checkb, EINA_FALSE);	
		elm_object_disabled_set(settings->navigationbar_checkb, EINA_FALSE);
	}
}




static void _allgroupselection_bt_clicked_cb(void *data, Evas_Object *obj, void *event_info __UNUSED__)
{
    Eina_List *l;
    Eina_List *l2;
    Elm_Object_Item *it;
    Evas_Object *li = data;

    l = elm_list_items_get(li);

   	EINA_LIST_FOREACH(l, l2, it)
        elm_list_item_selected_set(it, elm_check_state_get(obj));
}


static void _groupselection_bt_clicked_cb(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
{
    const char *s;
    Elm_Object_Item *it;

    s = elm_object_item_text_get(event_info);

    if(strcmp(s, _("All groups")) ==0)
        return;

    if(strncmp(elm_object_item_text_get(event_info), _("Group"), 5) !=0)
        return;

    it = event_info;
    while((it = elm_list_item_next(it)))
    {
        if(strncmp(elm_object_item_text_get(it), _("Group"), 5) ==0)
        break;

        elm_list_item_selected_set(it, EINA_TRUE);
    }
}




static void _update_module_settings()
{
    int i;  
    Elm_Object_Item *it = NULL;
       
    if(!settings->patient_selected)
        return;
        
    settings->navbar_wanted = EINA_TRUE;
    if((i = file_setting_int_read(settings->patient_selected->filename,"DIAPORAMA/navbar_wanted")) != -1)
        settings->navbar_wanted = i;

    settings->semantics_wanted = EINA_TRUE;
    if((i = file_setting_int_read(settings->patient_selected->filename,"DIAPORAMA/semantics_wanted")) != -1)
        settings->semantics_wanted = i;
        
    settings->graphemics_wanted = EINA_TRUE;
    if((i = file_setting_int_read(settings->patient_selected->filename,"DIAPORAMA/graphemics_wanted")) != -1)
        settings->graphemics_wanted = i;        

    settings->visuals_wanted = EINA_FALSE;
    if((i = file_setting_int_read(settings->patient_selected->filename, "DIAPORAMA/visuals_wanted")) != -1)
        settings->visuals_wanted = i;

    settings->itemlabel_wanted = EINA_FALSE;
    if((i = file_setting_int_read(settings->patient_selected->filename,"DIAPORAMA/itemlabel_wanted")) != -1)
            settings->itemlabel_wanted =  i;

    if((settings->nbserial_wanted = file_setting_int_read(settings->patient_selected->filename,"DIAPORAMA/nbserial_wanted")) == -1)
    	settings->nbserial_wanted = 10;

    settings->timers_wanted = EINA_FALSE;
    if((i = file_setting_int_read(settings->patient_selected->filename,"DIAPORAMA/timers_wanted")) != -1)
        settings->timers_wanted = i;
    if(settings->timers_wanted == EINA_TRUE)
    {   
        if((settings->exposition_timer_wanted = file_setting_int_read(settings->patient_selected->filename,"DIAPORAMA/exposition_timer_wanted")) == -1)
    	settings->exposition_timer_wanted = 15;

        if((settings->hiding_timer_wanted = file_setting_int_read(settings->patient_selected->filename,"DIAPORAMA/hiding_timer_wanted")) == -1)
    	settings->hiding_timer_wanted = 5;
    }
    else
    {
        elm_object_disabled_set(settings->exposition_sl, EINA_TRUE);
        elm_object_disabled_set(settings->hiding_sl, EINA_TRUE);
    }

    if((settings->imglvl_wanted = file_setting_int_read(settings->patient_selected->filename,"DIAPORAMA/imglvl_wanted")) == -1)
    	settings->imglvl_wanted = 0;
    	
	elm_check_state_set(settings->graphemics_checkb, settings->graphemics_wanted);
	elm_check_state_set(settings->visuals_checkb, settings->visuals_wanted);
	elm_check_state_set(settings->semantics_checkb, settings->semantics_wanted);
	elm_check_state_set(settings->timers_checkb, settings->timers_wanted);
	elm_slider_value_set(settings->exposition_sl, settings->exposition_timer_wanted);
	elm_slider_value_set(settings->hiding_sl, settings->hiding_timer_wanted);
	elm_slider_value_set(settings->nbserial_sl, settings->nbserial_wanted);
	elm_check_state_set(settings->navigationbar_checkb, settings->navbar_wanted);
	elm_check_state_set(settings->itemlabel_checkb, settings->itemlabel_wanted);


    switch(settings->imglvl_wanted)
    {
        case 0: 
                it = elm_segment_control_item_get(settings->imglvl_sc, 0);
                break;
        case 1:
                it = elm_segment_control_item_get(settings->imglvl_sc, 1);        
                break;
    }
    if(it)
        elm_segment_control_item_selected_set(it, EINA_TRUE);	    	
}


//
//
//
EAPI void _module_patient_set(Patient_Info *patient)
{
    if(patient)
    {
        INF(_("Set patient to %s %s"), patient->firstname, patient->lastname);
        settings->patient_selected = patient;
        _update_module_settings();
        elm_object_disabled_set(settings->exec_bt, EINA_FALSE);  
    }
    else
    {
        elm_object_disabled_set(settings->exec_bt, EINA_TRUE);
    }
}


//
//Module configuration panel.
//Configuration panel must return an elm layout widget and will be embbded in FriLogos
//MainUI.
//Params:
//@app:App_Info passed from main ui.
//
EAPI Evas_Object *_module_settings_get(App_Info *app)
{
    Evas_Object *gd, *frame, *bx, *table;
	Evas_Object *parent;
	Evas_Object *ic;
	Evas_Object *cb;

    //Set settings default or saved values.
	parent = app->win;
	settings->patient_selected = app->patient_selection;

    gd = elm_grid_add(parent);
    elm_grid_size_set(gd, 100, 100);
    elm_win_resize_object_add(parent, gd);
    evas_object_size_hint_weight_set(gd, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_show(gd);

    //Read all items and groups/subgroups and fill hoversel widget to allow user to select a group.
    settings->groups_list = elm_list_add(parent);
    elm_list_mode_set(settings->groups_list, ELM_LIST_SCROLL);
	elm_list_multi_select_set(settings->groups_list, EINA_TRUE);

	Eina_List * l = NULL;
	Eina_List *groups = groups_list_get();
	Group_Info *group;
	char t[512];
     Elm_Object_Item *it;
            
	cb = elm_check_add(parent);
    evas_object_smart_callback_add(cb, "changed", _allgroupselection_bt_clicked_cb, settings->groups_list);
    //evas_object_propagate_events_set(cb, 0);
    it = elm_list_item_append(settings->groups_list, _("All groups"), NULL, cb, NULL, settings->groups_list);
    elm_object_item_data_set(it, _("All groups"));

	EINA_LIST_FOREACH(groups, l, group)
	{
        snprintf(t, sizeof(t), _("Group %s"), group->name);
	    elm_list_item_append(settings->groups_list, elm_entry_markup_to_utf8(t), NULL, NULL, NULL, NULL);
	        
        char *s;
    	Eina_List * l2 = NULL;

        EINA_LIST_FOREACH(group->subgroups, l2, s)
        {
            snprintf(t, sizeof(t), "    %s", s);
	        it = elm_list_item_append(settings->groups_list, elm_entry_markup_to_utf8(t), NULL, NULL, NULL, NULL);
            snprintf(t, sizeof(t), "%s/%s", group->name, s);
	        elm_object_item_data_set(it, strdup(t));
        }
    }
    evas_object_smart_callback_add(settings->groups_list, "selected", _groupselection_bt_clicked_cb, settings);
    elm_list_go(settings->groups_list);
    evas_object_show(settings->groups_list);
    groups = groups_list_free(groups);

    frame= elm_frame_add(parent);
    elm_object_content_set(frame, settings->groups_list);
    elm_object_text_set(frame, _("Groups Selection"));
	elm_grid_pack(gd, frame , 0, 0, 40, 60);
    evas_object_show(frame);

    //Distractors settings.
    table = elm_table_add(parent);
    evas_object_show(table);
    elm_table_padding_set(table, 5, 5);    
    elm_table_homogeneous_set(table, EINA_TRUE);
   
	settings->graphemics_checkb = elm_check_add(parent);
	elm_object_text_set(settings->graphemics_checkb, _("Graphemics"));
    elm_table_pack(table, settings->graphemics_checkb, 0, 0, 1, 1);
	evas_object_show(settings->graphemics_checkb);
	evas_object_size_hint_weight_set(settings->graphemics_checkb, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(settings->graphemics_checkb, 0, 0);	
	evas_object_smart_callback_add(settings->graphemics_checkb, "changed",  _wantgraphemics_change_cb, settings);

	settings->visuals_checkb = elm_check_add(parent);
	elm_object_text_set(settings->visuals_checkb, _("Visual"));

    elm_table_pack(table, settings->visuals_checkb, 1, 0, 1, 1);
   	evas_object_show(settings->visuals_checkb);
	evas_object_size_hint_weight_set(settings->visuals_checkb, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(settings->visuals_checkb, 0, 0);	
	evas_object_smart_callback_add(settings->visuals_checkb, "changed",  _wantvisuals_change_cb, settings);

	settings->semantics_checkb = elm_check_add(parent);
	elm_object_text_set(settings->semantics_checkb, _("Semantics"));

    elm_table_pack(table, settings->semantics_checkb, 0, 1, 1, 1);
	evas_object_show(settings->semantics_checkb);
	evas_object_size_hint_weight_set(settings->semantics_checkb, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(settings->semantics_checkb, 0, 0);	
	evas_object_smart_callback_add(settings->semantics_checkb, "changed",  _wantsemantics_change_cb, settings);		

    frame= elm_frame_add(parent);
    elm_object_content_set(frame, table);
    elm_object_text_set(frame, _("Distractors"));
  	elm_grid_pack(gd, frame , 0, 61, 40, 30);
    evas_object_show(frame);
    //End of distractors settings.

    //Others settings.
    bx = elm_box_add(parent);
	evas_object_size_hint_weight_set(bx, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);    
    evas_object_show(bx);

	settings->timers_checkb = elm_check_add(parent);
    evas_object_size_hint_weight_set(settings->timers_checkb, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_text_set(settings->timers_checkb, _("Enable timers"));

	elm_grid_pack(gd, settings->timers_checkb, 1, 5, 10, 10);
    elm_box_pack_end(bx, settings->timers_checkb);
    evas_object_size_hint_align_set(settings->timers_checkb, 0, 0);
	evas_object_show(settings->timers_checkb);
	evas_object_smart_callback_add(settings->timers_checkb, "changed", _wanttimers_change_cb, settings);

	settings->exposition_sl = elm_slider_add(parent);
	elm_object_text_set(settings->exposition_sl, _("Exposition time:"));
	elm_slider_unit_format_set(settings->exposition_sl, _("%1.0f sec"));
	elm_slider_indicator_show_set(settings->exposition_sl, EINA_FALSE);
	elm_slider_min_max_set(settings->exposition_sl, 1, 60);
	elm_slider_span_size_set(settings->exposition_sl, 70);
	elm_slider_inverted_set(settings->exposition_sl, EINA_FALSE);
    elm_box_pack_end(bx, settings->exposition_sl);
    evas_object_size_hint_align_set(settings->exposition_sl, 0, 0);
	evas_object_show(settings->exposition_sl);
	evas_object_smart_callback_add(settings->exposition_sl, "changed", _showtimer_sl_changed_cb, settings);

	settings->hiding_sl = elm_slider_add(parent);
	elm_object_text_set(settings->hiding_sl, _("Hiding time:"));
	elm_slider_unit_format_set(settings->hiding_sl, _("%1.0f sec"));
	elm_slider_indicator_show_set(settings->hiding_sl, EINA_FALSE);
	elm_slider_min_max_set(settings->hiding_sl, 1, 60);
	elm_slider_span_size_set(settings->hiding_sl, 70);
	elm_slider_inverted_set(settings->hiding_sl, EINA_FALSE);
    elm_box_pack_end(bx, settings->hiding_sl);
    evas_object_size_hint_align_set(settings->hiding_sl, 0, 0);
	evas_object_show(settings->hiding_sl);
	evas_object_smart_callback_add(settings->hiding_sl, "changed", _hidetimer_sl_changed_cb, settings);

	settings->nbserial_sl = elm_slider_add(parent);
	elm_object_text_set(settings->nbserial_sl, _("Sequences:"));
	elm_slider_unit_format_set(settings->nbserial_sl, _("%1.0f"));
	elm_slider_indicator_show_set(settings->nbserial_sl, EINA_FALSE);
	elm_slider_min_max_set(settings->nbserial_sl, 1, 50);
	elm_slider_span_size_set(settings->nbserial_sl, 70);
	elm_slider_inverted_set(settings->nbserial_sl, EINA_FALSE);
	evas_object_show(settings->nbserial_sl);
    evas_object_size_hint_align_set(settings->nbserial_sl, 0, 0);	
	elm_box_pack_end(bx, settings->nbserial_sl);
	evas_object_smart_callback_add(settings->nbserial_sl, "changed", _nbserial_sl_changed_cb, settings);

	settings->itemlabel_checkb = elm_check_add(parent);
	elm_object_text_set(settings->itemlabel_checkb, _("Show item name"));
    evas_object_size_hint_align_set(settings->itemlabel_checkb, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_show(settings->itemlabel_checkb);
	elm_box_pack_end(bx, settings->itemlabel_checkb);
	evas_object_smart_callback_add(settings->itemlabel_checkb, "changed",  _wantitemname_change_cb, settings);

	settings->navigationbar_checkb = elm_check_add(parent);
	elm_object_text_set(settings->navigationbar_checkb, _("Navigation bar"));
    evas_object_size_hint_align_set(settings->navigationbar_checkb, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_show(settings->navigationbar_checkb);
	elm_box_pack_end(bx, settings->navigationbar_checkb);	
	evas_object_smart_callback_add(settings->navigationbar_checkb , "changed", _wantnavbar_change_cb, settings);

    settings->imglvl_sc = elm_segment_control_add(parent);
	elm_box_pack_end(bx,  settings->imglvl_sc);	
    evas_object_show( settings->imglvl_sc);
	evas_object_smart_callback_add( settings->imglvl_sc , "changed", _imglvl_sc_changed_cb, settings);        
    elm_segment_control_item_add( settings->imglvl_sc, NULL, _("Realistic"));
    elm_segment_control_item_add( settings->imglvl_sc, NULL, _("Abstract"));
        
    frame= elm_frame_add(parent);
    elm_object_content_set(frame, bx);
    elm_object_text_set(frame, _("Options"));
	elm_grid_pack(gd, frame , 41, 0, 40, 90);
    evas_object_show(frame);
    //End of others settings.

	settings->exec_bt = elm_button_add(parent);
	ic = elm_icon_add(parent);
   	elm_icon_order_lookup_set(ic, ELM_ICON_LOOKUP_FDO_THEME);
   	elm_icon_standard_set(ic, "exec");
   	elm_object_part_content_set(settings->exec_bt, "icon", ic);
	elm_object_text_set(settings->exec_bt, _("Start"));
	elm_grid_pack(gd, settings->exec_bt , 0, 84, 100, 12);
    elm_object_disabled_set(settings->exec_bt, EINA_TRUE);
	evas_object_smart_callback_add(settings->exec_bt, "clicked", _module_screen_create_cb, settings);
    evas_object_show(settings->exec_bt);

	return gd;
}



static Eina_Bool _module_init(void)
{
    INF(_("Alloc module ressources."));
    
    if(settings)
        return EINA_FALSE;

    settings = malloc(sizeof(*settings));

   return EINA_TRUE;
}


void _module_shutdown(void)
{
    INF(_("Free module ressources."));
    free(settings);    
    settings = NULL;
}


EINA_MODULE_INIT(_module_init);
EINA_MODULE_SHUTDOWN(_module_shutdown);
